#!/usr/bin/python

# Copyright: (c) 2016, Kamil Szczygiel <kamil.szczygiel () intel.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: influxdb_database
short_description: Manage InfluxDB databases
description:
    - Manage InfluxDB databases.
author: "Kamil Szczygiel (@kamsz)"
requirements:
    - "python >= 2.6"
    - "influxdb >= 0.9"
    - requests
options:
    database_name:
        description:
            - Name of the database.
        required: true
        type: str
    state:
        description:
            - Determines if the database should be created or destroyed.
        choices: [ absent, present ]
        default: present
        type: str
extends_documentation_fragment:
- community.general.influxdb

'''

EXAMPLES = r'''
# Example influxdb_database command from Ansible Playbooks
- name: Create database
  community.general.influxdb_database:
      hostname: "{{influxdb_ip_address}}"
      database_name: "{{influxdb_database_name}}"

- name: Destroy database
  community.general.influxdb_database:
      hostname: "{{influxdb_ip_address}}"
      database_name: "{{influxdb_database_name}}"
      state: absent

- name: Create database using custom credentials
  community.general.influxdb_database:
      hostname: "{{influxdb_ip_address}}"
      username: "{{influxdb_username}}"
      password: "{{influxdb_password}}"
      database_name: "{{influxdb_database_name}}"
      ssl: yes
      validate_certs: yes
'''

RETURN = r'''
# only defaults
'''

try:
    import requests.exceptions
    from influxdb import exceptions
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.influxdb import InfluxDb


def find_database(module, client, database_name):
    database = None

    try:
        databases = client.get_list_database()
        for db in databases:
            if db['name'] == database_name:
                database = db
                break
    except requests.exceptions.ConnectionError as e:
        module.fail_json(msg=str(e))
    return database


def create_database(module, client, database_name):
    if not module.check_mode:
        try:
            client.create_database(database_name)
        except requests.exceptions.ConnectionError as e:
            module.fail_json(msg=str(e))

    module.exit_json(changed=True)


def drop_database(module, client, database_name):
    if not module.check_mode:
        try:
            client.drop_database(database_name)
        except exceptions.InfluxDBClientError as e:
            module.fail_json(msg=e.content)

    module.exit_json(changed=True)


def main():
    argument_spec = InfluxDb.influxdb_argument_spec()
    argument_spec.update(
        database_name=dict(required=True, type='str'),
        state=dict(default='present', type='str', choices=['present', 'absent'])
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True
    )

    state = module.params['state']

    influxdb = InfluxDb(module)
    client = influxdb.connect_to_influxdb()
    database_name = influxdb.database_name
    database = find_database(module, client, database_name)

    if state == 'present':
        if database:
            module.exit_json(changed=False)
        else:
            create_database(module, client, database_name)

    if state == 'absent':
        if database:
            drop_database(module, client, database_name)
        else:
            module.exit_json(changed=False)


if __name__ == '__main__':
    main()



# (c) 2020 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


from ansible_collections.community.internal_test_tools.tests.unit.utils.fetch_url_module_framework import (
    FetchUrlCall,
    BaseTestModule,
)

from ansible_collections.community.general.plugins.module_utils.hetzner import BASE_URL
from ansible_collections.community.general.plugins.modules.net_tools import hetzner_failover_ip


class TestHetznerFailoverIP(BaseTestModule):
    MOCK_ANSIBLE_MODULEUTILS_BASIC_ANSIBLEMODULE = 'ansible_collections.community.general.plugins.modules.net_tools.hetzner_failover_ip.AnsibleModule'
    MOCK_ANSIBLE_MODULEUTILS_URLS_FETCH_URL = 'ansible_collections.community.general.plugins.module_utils.hetzner.fetch_url'

    # Tests for state idempotence (routed and unrouted)

    def test_unrouted(self, mocker):
        result = self.run_module_success(mocker, hetzner_failover_ip, {
            'hetzner_user': '',
            'hetzner_password': '',
            'failover_ip': '1.2.3.4',
            'state': 'unrouted',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': None,
                },
            })
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['value'] is None
        assert result['state'] == 'unrouted'

    def test_routed(self, mocker):
        result = self.run_module_success(mocker, hetzner_failover_ip, {
            'hetzner_user': '',
            'hetzner_password': '',
            'failover_ip': '1.2.3.4',
            'state': 'routed',
            'value': '4.3.2.1',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': '4.3.2.1',
                },
            })
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['value'] == '4.3.2.1'
        assert result['state'] == 'routed'

    # Tests for changing state (unrouted to routed, vice versa)

    def test_unrouted_to_routed(self, mocker):
        result = self.run_module_success(mocker, hetzner_failover_ip, {
            'hetzner_user': '',
            'hetzner_password': '',
            'failover_ip': '1.2.3.4',
            'state': 'routed',
            'value': '4.3.2.1',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': None,
                },
            })
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
            FetchUrlCall('POST', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': '4.3.2.1',
                },
            })
            .expect_form_value('active_server_ip', '4.3.2.1')
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is True
        assert result['value'] == '4.3.2.1'
        assert result['state'] == 'routed'

    def test_routed_to_unrouted(self, mocker):
        result = self.run_module_success(mocker, hetzner_failover_ip, {
            'hetzner_user': '',
            'hetzner_password': '',
            'failover_ip': '1.2.3.4',
            'state': 'unrouted',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': '4.3.2.1',
                },
            })
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
            FetchUrlCall('DELETE', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': None,
                },
            })
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is True
        assert result['value'] is None
        assert result['state'] == 'unrouted'

    # Tests for re-routing

    def test_rerouting(self, mocker):
        result = self.run_module_success(mocker, hetzner_failover_ip, {
            'hetzner_user': '',
            'hetzner_password': '',
            'failover_ip': '1.2.3.4',
            'state': 'routed',
            'value': '4.3.2.1',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': '5.4.3.2',
                },
            })
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
            FetchUrlCall('POST', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': '4.3.2.1',
                },
            })
            .expect_form_value('active_server_ip', '4.3.2.1')
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is True
        assert result['value'] == '4.3.2.1'
        assert result['state'] == 'routed'

    def test_rerouting_already_routed(self, mocker):
        result = self.run_module_success(mocker, hetzner_failover_ip, {
            'hetzner_user': '',
            'hetzner_password': '',
            'failover_ip': '1.2.3.4',
            'state': 'routed',
            'value': '4.3.2.1',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': '5.4.3.2',
                },
            })
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
            FetchUrlCall('POST', 409)
            .result_json({
                'error': {
                    'status': 409,
                    'code': 'FAILOVER_ALREADY_ROUTED',
                    'message': 'Failover already routed',
                },
                'failover': {
                    'ip': '1.2.3.4',
                    'netmask': '255.255.255.255',
                    'server_ip': '2.3.4.5',
                    'server_number': 2345,
                    'active_server_ip': '4.3.2.1',
                },
            })
            .expect_form_value('active_server_ip', '4.3.2.1')
            .expect_url('{0}/failover/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['value'] == '4.3.2.1'
        assert result['state'] == 'routed'



#!/usr/bin/python
#
# Copyright: (c) Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: jenkins_job
short_description: Manage jenkins jobs
description:
    - Manage Jenkins jobs by using Jenkins REST API.
requirements:
  - "python-jenkins >= 0.4.12"
author: "Sergio Millan Rodriguez (@sermilrod)"
options:
  config:
    type: str
    description:
      - config in XML format.
      - Required if job does not yet exist.
      - Mutually exclusive with C(enabled).
      - Considered if C(state=present).
    required: false
  enabled:
    description:
      - Whether the job should be enabled or disabled.
      - Mutually exclusive with C(config).
      - Considered if C(state=present).
    type: bool
    required: false
  name:
    type: str
    description:
      - Name of the Jenkins job.
    required: true
  password:
    type: str
    description:
      - Password to authenticate with the Jenkins server.
    required: false
  state:
    type: str
    description:
      - Attribute that specifies if the job has to be created or deleted.
    required: false
    default: present
    choices: ['present', 'absent']
  token:
    type: str
    description:
      - API token used to authenticate alternatively to password.
    required: false
  url:
    type: str
    description:
      - URL where the Jenkins server is accessible.
    required: false
    default: http://localhost:8080
  user:
    type: str
    description:
       - User to authenticate with the Jenkins server.
    required: false
'''

EXAMPLES = '''
- name: Create a jenkins job using basic authentication
  community.general.jenkins_job:
    config: "{{ lookup('file', 'templates/test.xml') }}"
    name: test
    password: admin
    url: http://localhost:8080
    user: admin

- name: Create a jenkins job using the token
  community.general.jenkins_job:
    config: "{{ lookup('template', 'templates/test.xml.j2') }}"
    name: test
    token: asdfasfasfasdfasdfadfasfasdfasdfc
    url: http://localhost:8080
    user: admin

- name: Delete a jenkins job using basic authentication
  community.general.jenkins_job:
    name: test
    password: admin
    state: absent
    url: http://localhost:8080
    user: admin

- name: Delete a jenkins job using the token
  community.general.jenkins_job:
    name: test
    token: asdfasfasfasdfasdfadfasfasdfasdfc
    state: absent
    url: http://localhost:8080
    user: admin

- name: Disable a jenkins job using basic authentication
  community.general.jenkins_job:
    name: test
    password: admin
    enabled: False
    url: http://localhost:8080
    user: admin

- name: Disable a jenkins job using the token
  community.general.jenkins_job:
    name: test
    token: asdfasfasfasdfasdfadfasfasdfasdfc
    enabled: False
    url: http://localhost:8080
    user: admin
'''

RETURN = '''
---
name:
  description: Name of the jenkins job.
  returned: success
  type: str
  sample: test-job
state:
  description: State of the jenkins job.
  returned: success
  type: str
  sample: present
enabled:
  description: Whether the jenkins job is enabled or not.
  returned: success
  type: bool
  sample: true
user:
  description: User used for authentication.
  returned: success
  type: str
  sample: admin
url:
  description: Url to connect to the Jenkins server.
  returned: success
  type: str
  sample: https://jenkins.mydomain.com
'''

import traceback
import xml.etree.ElementTree as ET

JENKINS_IMP_ERR = None
try:
    import jenkins
    python_jenkins_installed = True
except ImportError:
    JENKINS_IMP_ERR = traceback.format_exc()
    python_jenkins_installed = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils._text import to_native


class JenkinsJob:

    def __init__(self, module):
        self.module = module

        self.config = module.params.get('config')
        self.name = module.params.get('name')
        self.password = module.params.get('password')
        self.state = module.params.get('state')
        self.enabled = module.params.get('enabled')
        self.token = module.params.get('token')
        self.user = module.params.get('user')
        self.jenkins_url = module.params.get('url')
        self.server = self.get_jenkins_connection()

        self.result = {
            'changed': False,
            'url': self.jenkins_url,
            'name': self.name,
            'user': self.user,
            'state': self.state,
            'diff': {
                'before': "",
                'after': ""
            }
        }

        self.EXCL_STATE = "excluded state"

    def get_jenkins_connection(self):
        try:
            if (self.user and self.password):
                return jenkins.Jenkins(self.jenkins_url, self.user, self.password)
            elif (self.user and self.token):
                return jenkins.Jenkins(self.jenkins_url, self.user, self.token)
            elif (self.user and not (self.password or self.token)):
                return jenkins.Jenkins(self.jenkins_url, self.user)
            else:
                return jenkins.Jenkins(self.jenkins_url)
        except Exception as e:
            self.module.fail_json(msg='Unable to connect to Jenkins server, %s' % to_native(e), exception=traceback.format_exc())

    def get_job_status(self):
        try:
            response = self.server.get_job_info(self.name)
            if "color" not in response:
                return self.EXCL_STATE
            else:
                return to_native(response['color'])

        except Exception as e:
            self.module.fail_json(msg='Unable to fetch job information, %s' % to_native(e), exception=traceback.format_exc())

    def job_exists(self):
        try:
            return bool(self.server.job_exists(self.name))
        except Exception as e:
            self.module.fail_json(msg='Unable to validate if job exists, %s for %s' % (to_native(e), self.jenkins_url),
                                  exception=traceback.format_exc())

    def get_config(self):
        return job_config_to_string(self.config)

    def get_current_config(self):
        return job_config_to_string(self.server.get_job_config(self.name).encode('utf-8'))

    def has_config_changed(self):
        # config is optional, if not provided we keep the current config as is
        if self.config is None:
            return False

        config_file = self.get_config()
        machine_file = self.get_current_config()

        self.result['diff']['after'] = config_file
        self.result['diff']['before'] = machine_file

        if machine_file != config_file:
            return True
        return False

    def present_job(self):
        if self.config is None and self.enabled is None:
            self.module.fail_json(msg='one of the following params is required on state=present: config,enabled')

        if not self.job_exists():
            self.create_job()
        else:
            self.update_job()

    def has_state_changed(self, status):
        # Keep in current state if enabled arg_spec is not given
        if self.enabled is None:
            return False

        if ((self.enabled is False and status != "disabled") or (self.enabled is True and status == "disabled")):
            return True
        return False

    def switch_state(self):
        if self.enabled is False:
            self.server.disable_job(self.name)
        else:
            self.server.enable_job(self.name)

    def update_job(self):
        try:
            status = self.get_job_status()

            # Handle job config
            if self.has_config_changed():
                self.result['changed'] = True
                if not self.module.check_mode:
                    self.server.reconfig_job(self.name, self.get_config())

            # Handle job disable/enable
            elif (status != self.EXCL_STATE and self.has_state_changed(status)):
                self.result['changed'] = True
                if not self.module.check_mode:
                    self.switch_state()

        except Exception as e:
            self.module.fail_json(msg='Unable to reconfigure job, %s for %s' % (to_native(e), self.jenkins_url),
                                  exception=traceback.format_exc())

    def create_job(self):
        if self.config is None:
            self.module.fail_json(msg='missing required param: config')

        self.result['changed'] = True
        try:
            config_file = self.get_config()
            self.result['diff']['after'] = config_file
            if not self.module.check_mode:
                self.server.create_job(self.name, config_file)
        except Exception as e:
            self.module.fail_json(msg='Unable to create job, %s for %s' % (to_native(e), self.jenkins_url),
                                  exception=traceback.format_exc())

    def absent_job(self):
        if self.job_exists():
            self.result['changed'] = True
            self.result['diff']['before'] = self.get_current_config()
            if not self.module.check_mode:
                try:
                    self.server.delete_job(self.name)
                except Exception as e:
                    self.module.fail_json(msg='Unable to delete job, %s for %s' % (to_native(e), self.jenkins_url),
                                          exception=traceback.format_exc())

    def get_result(self):
        result = self.result
        if self.job_exists():
            result['enabled'] = self.get_job_status() != "disabled"
        else:
            result['enabled'] = None
        return result


def test_dependencies(module):
    if not python_jenkins_installed:
        module.fail_json(
            msg=missing_required_lib("python-jenkins",
                                     url="https://python-jenkins.readthedocs.io/en/latest/install.html"),
            exception=JENKINS_IMP_ERR)


def job_config_to_string(xml_str):
    return ET.tostring(ET.fromstring(xml_str)).decode('ascii')


def main():
    module = AnsibleModule(
        argument_spec=dict(
            config=dict(type='str', required=False),
            name=dict(type='str', required=True),
            password=dict(type='str', required=False, no_log=True),
            state=dict(type='str', required=False, choices=['present', 'absent'], default="present"),
            enabled=dict(required=False, type='bool'),
            token=dict(type='str', required=False, no_log=True),
            url=dict(type='str', required=False, default="http://localhost:8080"),
            user=dict(type='str', required=False)
        ),
        mutually_exclusive=[
            ['password', 'token'],
            ['config', 'enabled'],
        ],
        supports_check_mode=True,
    )

    test_dependencies(module)
    jenkins_job = JenkinsJob(module)

    if module.params.get('state') == "present":
        jenkins_job.present_job()
    else:
        jenkins_job.absent_job()

    result = jenkins_job.get_result()
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# (c) 2017 Red Hat, Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type

import re

import pytest

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import (
    config,
)


RUNNING = """interface Ethernet1
   speed auto
   no switchport
   no lldp receive
!
interface Ethernet2
   speed auto
   no switchport
   no lldp transmit
!
interface Management1
   ip address dhcp
!"""

WANT_SRC_1 = """
router ospfv3
   fips restrictions
   address-family ipv4
      fips restrictions
      redistribute connected
      exit
   address-family ipv6
      router-id 10.1.1.1
      exit
   exit
router ospfv3 vrf vrf01
   fips restrictions
   address-family ipv4
      passive-interface default
      exit
   address-family ipv6
      fips restrictions
      exit
   exit
router ospfv3 vrf vrf02
   fips restrictions
   address-family ipv6
      fips restrictions
      exit
   exit
"""

HAVE_SRC_1 = ""

WANT_SRC_2 = """
interface loopback0
 ip ospf 1 area 0.0.0.0
!
interface GigabitEthernet0/1
 ip ospf 1 area 0.0.0.0
interface GigabitEthernet0/2
 ip ospf 1 area 0.0.0.0
!
router isis fabric
 is-type level-2
 net 49.0000.0000.0003.00
!
interface loopback0
 ip router isis fabric
!
interface GigabitEthernet0/1
 ip router isis fabric
interface GigabitEthernet0/2
 ip router isis fabric
"""

HAVE_SRC_2 = """
interface GigabitEthernet0/1
interface GigabitEthernet0/2
"""

ORIGINAL_DEFAULT_IGNORE_LINES_RE = config.DEFAULT_IGNORE_LINES_RE.copy()


def test_config_items():
    net_config = config.NetworkConfig(indent=3, contents=RUNNING)
    assert len(net_config.items) == 10

    net_config = config.NetworkConfig(
        indent=3, contents=RUNNING, ignore_lines=[r"\s*no .*"]
    )
    assert len(net_config.items) == 6

    net_config = config.NetworkConfig(
        indent=3, contents=RUNNING, ignore_lines=[re.compile(r"\s*no .*")]
    )
    assert len(net_config.items) == 6
    # above updates the global DEFAULT_IGNORE_LINES_RE list, revert back the global
    # to it's original value
    config.DEFAULT_IGNORE_LINES_RE = ORIGINAL_DEFAULT_IGNORE_LINES_RE


def test_config_get_block():
    net_config = config.NetworkConfig(indent=3, contents=RUNNING)

    with pytest.raises(
        AssertionError, match="path argument must be a list object"
    ):
        net_config.get_block("interface Ethernet2")

    with pytest.raises(ValueError, match="path does not exist in config"):
        net_config.get_block(["interface Ethernet3"])

    block = net_config.get_block(["interface Ethernet2"])
    assert len(block) == 4


def test_line_hierarchy():
    net_config = config.NetworkConfig(indent=3, contents=RUNNING)

    lines = net_config.items
    assert lines[0].has_children
    assert not lines[0].has_parents
    assert not lines[1].has_children
    assert lines[1].has_parents


def test_updates_repeat_lines():
    candidate_obj = config.NetworkConfig(indent=3)
    candidate_obj.load(WANT_SRC_1)

    running_obj = config.NetworkConfig(indent=3, contents=HAVE_SRC_1)

    configdiffobjs = candidate_obj.difference(running_obj)
    diff_list = config.dumps(configdiffobjs, "commands").split("\n")
    want_src_list = WANT_SRC_1.strip().split("\n")
    for generated_diff_line, candidate_diff_line in zip(
        diff_list, want_src_list
    ):
        assert generated_diff_line == candidate_diff_line.strip()


def test_updates_repeat_parents():
    expected_diff = [
        "interface loopback0",
        "ip ospf 1 area 0.0.0.0",
        "interface GigabitEthernet0/1",
        "ip ospf 1 area 0.0.0.0",
        "interface GigabitEthernet0/2",
        "ip ospf 1 area 0.0.0.0",
        "router isis fabric",
        "is-type level-2",
        "net 49.0000.0000.0003.00",
        "interface loopback0",
        "ip router isis fabric",
        "interface GigabitEthernet0/1",
        "ip router isis fabric",
        "interface GigabitEthernet0/2",
        "ip router isis fabric",
    ]
    candidate_obj = config.NetworkConfig(indent=1)
    candidate_obj.load(WANT_SRC_2)

    running_obj = config.NetworkConfig(indent=1, contents=HAVE_SRC_2)

    configdiffobjs = candidate_obj.difference(running_obj)
    diff_list = config.dumps(configdiffobjs, "commands").split("\n")

    for generated_diff_line, candidate_diff_line in zip(
        diff_list, expected_diff
    ):
        print(generated_diff_line, candidate_diff_line)
        assert generated_diff_line == candidate_diff_line.strip()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios_lldp_global class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    to_list,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import (
    Facts,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.utils.utils import (
    dict_to_set,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.utils.utils import (
    filter_dict_having_none_value,
)


class Lldp_global(ConfigBase):
    """
    The ios_lldp_global class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["lldp_global"]

    tlv_select_params = {
        "four_wire_power_management": "4-wire-power-management",
        "mac_phy_cfg": "mac-phy-cfg",
        "management_address": "management-address",
        "port_description": "port-description",
        "port_vlan": "port-vlan",
        "power_management": "power-management",
        "system_capabilities": "system-capabilities",
        "system_description": "system-description",
        "system_name": "system-name",
    }

    def __init__(self, module):
        super(Lldp_global, self).__init__(module)

    def get_lldp_global_facts(self, data=None):
        """ Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset, self.gather_network_resources, data=data
        )
        lldp_global_facts = facts["ansible_network_resources"].get(
            "lldp_global"
        )
        if not lldp_global_facts:
            return {}

        return lldp_global_facts

    def execute_module(self):
        """ Execute the module

        :rtype: A dictionary
        :returns: The result from moduel execution
        """
        result = {"changed": False}
        commands = list()
        warnings = list()

        if self.state in self.ACTION_STATES:
            existing_lldp_global_facts = self.get_lldp_global_facts()
        else:
            existing_lldp_global_facts = dict()

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_lldp_global_facts))
        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True
        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_lldp_global_facts = self.get_lldp_global_facts()
        elif self.state == "rendered":
            result["rendered"] = commands
        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed"
                )
            result["parsed"] = self.get_lldp_global_facts(data=running_config)
        else:
            changed_lldp_global_facts = dict()

        if self.state in self.ACTION_STATES:
            result["before"] = existing_lldp_global_facts
            if result["changed"]:
                result["after"] = changed_lldp_global_facts
        elif self.state == "gathered":
            result["gathered"] = changed_lldp_global_facts
        result["warnings"] = warnings

        return result

    def set_config(self, existing_lldp_global_facts):
        """ Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the deisred configuration
        """
        want = self._module.params["config"]
        have = existing_lldp_global_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """ Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the deisred configuration
        """
        commands = []
        if self.state in ("merged", "replaced", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    self.state
                )
            )

        if self.state == "overridden":
            commands = self._state_overridden(want, have)
        elif self.state == "deleted":
            commands = self._state_deleted(want, have)
        elif self.state in ("merged", "rendered"):
            commands = self._state_merged(want, have)
        elif self.state == "replaced":
            commands = self._state_replaced(want, have)

        return commands

    def _state_replaced(self, want, have):
        """ The command generator when state is replaced

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the deisred configuration
        """
        commands = []

        have_dict = filter_dict_having_none_value(want, have)
        commands.extend(self._clear_config(have_dict))
        commands.extend(self._set_config(want, have))

        return commands

    def _state_merged(self, want, have):
        """ The command generator when state is merged

        :param want: the additive configuration as a dictionary
        :param obj_in_have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []

        commands.extend(self._set_config(want, have))

        return commands

    def _state_deleted(self, want, have):
        """ The command generator when state is deleted

        :param want: the objects from which the configuration should be removed
        :param obj_in_have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []

        commands.extend(self._clear_config(have))

        return commands

    def _remove_command_from_config_list(self, cmd, commands):
        if cmd not in commands:
            commands.append("no %s" % cmd)

    def add_command_to_config_list(self, cmd, commands):
        if cmd not in commands:
            commands.append(cmd)

    def _set_config(self, want, have):
        # Set the lldp global config based on the want and have config
        commands = []

        # Get the diff b/w want and have
        want_dict = dict_to_set(want)
        have_dict = dict_to_set(have)
        diff = want_dict - have_dict

        if diff:
            diff = dict(diff)
            holdtime = diff.get("holdtime")
            enabled = diff.get("enabled")
            timer = diff.get("timer")
            reinit = diff.get("reinit")
            tlv_select = diff.get("tlv_select")

            if holdtime:
                cmd = "lldp holdtime {0}".format(holdtime)
                self.add_command_to_config_list(cmd, commands)
            if enabled:
                cmd = "lldp run"
                self.add_command_to_config_list(cmd, commands)
            if timer:
                cmd = "lldp timer {0}".format(timer)
                self.add_command_to_config_list(cmd, commands)
            if reinit:
                cmd = "lldp reinit {0}".format(reinit)
                self.add_command_to_config_list(cmd, commands)
            if tlv_select:
                tlv_selec_dict = dict(tlv_select)
                for k, v in iteritems(self.tlv_select_params):
                    if k in tlv_selec_dict and tlv_selec_dict[k]:
                        cmd = "lldp tlv-select {0}".format(v)
                        self.add_command_to_config_list(cmd, commands)

        return commands

    def _clear_config(self, have):
        # Delete the lldp global config based on the want and have config
        commands = []

        if have.get("holdtime"):
            cmd = "lldp holdtime"
            self._remove_command_from_config_list(cmd, commands)
        if have.get("enabled"):
            cmd = "lldp run"
            self._remove_command_from_config_list(cmd, commands)
        if have.get("timer"):
            cmd = "lldp timer"
            self._remove_command_from_config_list(cmd, commands)
        if have.get("reinit"):
            cmd = "lldp reinit"
            self._remove_command_from_config_list(cmd, commands)

        return commands



# (c) 2019 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


from ansible_collections.community.internal_test_tools.tests.unit.utils.fetch_url_module_framework import (
    FetchUrlCall,
    BaseTestModule,
)

from ansible_collections.community.hrobot.plugins.module_utils.robot import BASE_URL
from ansible_collections.community.hrobot.plugins.modules import firewall_info


class TestHetznerFirewallInfo(BaseTestModule):
    MOCK_ANSIBLE_MODULEUTILS_BASIC_ANSIBLEMODULE = 'ansible_collections.community.hrobot.plugins.modules.firewall_info.AnsibleModule'
    MOCK_ANSIBLE_MODULEUTILS_URLS_FETCH_URL = 'ansible_collections.community.hrobot.plugins.module_utils.robot.fetch_url'

    # Tests for state (absent and present)

    def test_absent(self, mocker):
        result = self.run_module_success(mocker, firewall_info, {
            'hetzner_user': '',
            'hetzner_password': '',
            'server_ip': '1.2.3.4',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'disabled',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['firewall']['status'] == 'disabled'
        assert result['firewall']['server_ip'] == '1.2.3.4'
        assert result['firewall']['server_number'] == 1

    def test_absent_no_rules(self, mocker):
        result = self.run_module_success(mocker, firewall_info, {
            'hetzner_user': '',
            'hetzner_password': '',
            'server_ip': '1.2.3.4',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'disabled',
                    'whitelist_hos': False,
                    'port': 'main',
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['firewall']['status'] == 'disabled'
        assert result['firewall']['server_ip'] == '1.2.3.4'
        assert result['firewall']['server_number'] == 1
        assert 'rules' in result['firewall']
        assert 'input' in result['firewall']['rules']
        assert len(result['firewall']['rules']['input']) == 0

    def test_present(self, mocker):
        result = self.run_module_success(mocker, firewall_info, {
            'hetzner_user': '',
            'hetzner_password': '',
            'server_ip': '1.2.3.4',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'active',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['firewall']['status'] == 'active'
        assert result['firewall']['server_ip'] == '1.2.3.4'
        assert result['firewall']['server_number'] == 1
        assert len(result['firewall']['rules']['input']) == 0

    def test_present_w_rules(self, mocker):
        result = self.run_module_success(mocker, firewall_info, {
            'hetzner_user': '',
            'hetzner_password': '',
            'server_ip': '1.2.3.4',
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'active',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [
                            {
                                'name': 'Accept HTTPS traffic',
                                'ip_version': 'ipv4',
                                'dst_ip': None,
                                'dst_port': '443',
                                'src_ip': None,
                                'src_port': None,
                                'protocol': 'tcp',
                                'tcp_flags': None,
                                'action': 'accept',
                            },
                            {
                                'name': None,
                                'ip_version': 'ipv4',
                                'dst_ip': None,
                                'dst_port': None,
                                'src_ip': None,
                                'src_port': None,
                                'protocol': None,
                                'tcp_flags': None,
                                'action': 'discard',
                            }
                        ],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['firewall']['status'] == 'active'
        assert result['firewall']['server_ip'] == '1.2.3.4'
        assert result['firewall']['server_number'] == 1
        assert len(result['firewall']['rules']['input']) == 2
        assert result['firewall']['rules']['input'][0]['name'] == 'Accept HTTPS traffic'
        assert result['firewall']['rules']['input'][0]['dst_port'] == '443'
        assert result['firewall']['rules']['input'][0]['action'] == 'accept'
        assert result['firewall']['rules']['input'][1]['dst_port'] is None
        assert result['firewall']['rules']['input'][1]['action'] == 'discard'

    # Tests for wait_for_configured in getting status

    def test_wait_get(self, mocker):
        mocker.patch('time.sleep', lambda duration: None)
        result = self.run_module_success(mocker, firewall_info, {
            'hetzner_user': '',
            'hetzner_password': '',
            'server_ip': '1.2.3.4',
            'wait_for_configured': True,
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'in process',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'in process',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'active',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['firewall']['status'] == 'active'
        assert result['firewall']['server_ip'] == '1.2.3.4'
        assert result['firewall']['server_number'] == 1

    def test_wait_get_timeout(self, mocker):
        mocker.patch('time.sleep', lambda duration: None)
        result = self.run_module_failed(mocker, firewall_info, {
            'hetzner_user': '',
            'hetzner_password': '',
            'server_ip': '1.2.3.4',
            'wait_for_configured': True,
            'timeout': 0,
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'in process',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'in process',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['msg'] == 'Timeout while waiting for firewall to be configured.'

    def test_nowait_get(self, mocker):
        result = self.run_module_success(mocker, firewall_info, {
            'hetzner_user': '',
            'hetzner_password': '',
            'server_ip': '1.2.3.4',
            'wait_for_configured': False,
        }, [
            FetchUrlCall('GET', 200)
            .result_json({
                'firewall': {
                    'server_ip': '1.2.3.4',
                    'server_number': 1,
                    'status': 'in process',
                    'whitelist_hos': False,
                    'port': 'main',
                    'rules': {
                        'input': [],
                    },
                },
            })
            .expect_url('{0}/firewall/1.2.3.4'.format(BASE_URL)),
        ])
        assert result['changed'] is False
        assert result['firewall']['status'] == 'in process'
        assert result['firewall']['server_ip'] == '1.2.3.4'
        assert result['firewall']['server_number'] == 1



#!/usr/bin/python

# Copyright: (c) 2018, Stephan Schwarz <stearz@gmx.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: utm_ca_host_key_cert

author:
    - Stephan Schwarz (@stearz)

short_description: create, update or destroy ca host_key_cert entry in Sophos UTM

description:
    - Create, update or destroy a ca host_key_cert entry in SOPHOS UTM.
    - This module needs to have the REST Ability of the UTM to be activated.


options:
    name:
        description:
          - The name of the object. Will be used to identify the entry.
        required: true
        type: str
    ca:
        description:
          - A reference to an existing utm_ca_signing_ca or utm_ca_verification_ca object.
        required: true
        type: str
    meta:
        description:
          - A reference to an existing utm_ca_meta_x509 object.
        required: true
        type: str
    certificate:
        description:
          - The certificate in PEM format.
        required: true
        type: str
    comment:
        description:
          - Optional comment string.
        type: str
    encrypted:
        description:
          - Optionally enable encryption.
        default: False
        type: bool
    key:
        description:
          - Optional private key in PEM format.
        type: str

extends_documentation_fragment:
- community.general.utm

'''

EXAMPLES = """
- name: Create a ca_host_key_cert entry
  community.general.utm_ca_host_key_cert:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestHostKeyCertEntry
    ca: REF_ca/signing_ca_OBJECT_STRING
    meta: REF_ca/meta_x509_OBJECT_STRING
    certificate: |
      --- BEGIN CERTIFICATE ---
      . . .
       . . .
      . . .
      --- END CERTIFICATE ---
    state: present

- name: Remove a ca_host_key_cert entry
  community.general.utm_ca_host_key_cert:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestHostKeyCertEntry
    state: absent

- name: Read a ca_host_key_cert entry
  community.general.utm_ca_host_key_cert:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestHostKeyCertEntry
    state: info

"""

RETURN = """
result:
    description: The utm object that was created
    returned: success
    type: complex
    contains:
        _ref:
            description: The reference name of the object
            type: str
        _locked:
            description: Whether or not the object is currently locked
            type: bool
        _type:
            description: The type of the object
            type: str
        name:
            description: The name of the object
            type: str
        ca:
            description: A reference to an existing utm_ca_signing_ca or utm_ca_verification_ca object.
            type: str
        meta:
            description: A reference to an existing utm_ca_meta_x509 object.
            type: str
        certificate:
            description: The certificate in PEM format
            type: str
        comment:
            description: Comment string (may be empty string)
            type: str
        encrypted:
            description: If encryption is enabled
            type: bool
        key:
            description: Private key in PEM format (may be empty string)
            type: str
"""

from ansible_collections.community.general.plugins.module_utils.utm_utils import UTM, UTMModule
from ansible.module_utils._text import to_native


def main():
    endpoint = "ca/host_key_cert"
    key_to_check_for_changes = ["ca", "certificate", "comment", "encrypted", "key", "meta"]
    module = UTMModule(
        argument_spec=dict(
            name=dict(type='str', required=True),
            ca=dict(type='str', required=True),
            meta=dict(type='str', required=True),
            certificate=dict(type='str', required=True),
            comment=dict(type='str', required=False),
            encrypted=dict(type='bool', required=False, default=False),
            key=dict(type='str', required=False, no_log=True),
        )
    )
    try:
        # This is needed because the bool value only accepts int values in the backend
        UTM(module, endpoint, key_to_check_for_changes).execute()
    except Exception as e:
        module.fail_json(msg=to_native(e))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2016, Adam Števko <adam.stevko@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: zpool_facts
short_description: Gather facts about ZFS pools.
description:
  - Gather facts from ZFS pool properties.
author: Adam Števko (@xen0l)
options:
    name:
        description:
            - ZFS pool name.
        aliases: [ "pool", "zpool" ]
        required: false
    parsable:
        description:
            - Specifies if property values should be displayed in machine
              friendly format.
        type: bool
        default: False
        required: false
    properties:
        description:
            - Specifies which dataset properties should be queried in comma-separated format.
              For more information about dataset properties, check zpool(1M) man page.
        default: all
        required: false
'''

EXAMPLES = '''
- name: Gather facts about ZFS pool rpool
  community.general.zpool_facts: pool=rpool

- name: Gather space usage about all imported ZFS pools
  community.general.zpool_facts: properties='free,size'

- name: Print gathered information
  ansible.builtin.debug:
    msg: 'ZFS pool {{ item.name }} has {{ item.free }} free space out of {{ item.size }}.'
  with_items: '{{ ansible_zfs_pools }}'
'''

RETURN = '''
ansible_facts:
    description: Dictionary containing all the detailed information about the ZFS pool facts
    returned: always
    type: complex
    contains:
        ansible_zfs_pools:
            description: ZFS pool facts
            returned: always
            type: str
            sample:
                {
                    "allocated": "3.46G",
                    "altroot": "-",
                    "autoexpand": "off",
                    "autoreplace": "off",
                    "bootfs": "rpool/ROOT/openindiana",
                    "cachefile": "-",
                    "capacity": "6%",
                    "comment": "-",
                    "dedupditto": "0",
                    "dedupratio": "1.00x",
                    "delegation": "on",
                    "expandsize": "-",
                    "failmode": "wait",
                    "feature@async_destroy": "enabled",
                    "feature@bookmarks": "enabled",
                    "feature@edonr": "enabled",
                    "feature@embedded_data": "active",
                    "feature@empty_bpobj": "active",
                    "feature@enabled_txg": "active",
                    "feature@extensible_dataset": "enabled",
                    "feature@filesystem_limits": "enabled",
                    "feature@hole_birth": "active",
                    "feature@large_blocks": "enabled",
                    "feature@lz4_compress": "active",
                    "feature@multi_vdev_crash_dump": "enabled",
                    "feature@sha512": "enabled",
                    "feature@skein": "enabled",
                    "feature@spacemap_histogram": "active",
                    "fragmentation": "3%",
                    "free": "46.3G",
                    "freeing": "0",
                    "guid": "15729052870819522408",
                    "health": "ONLINE",
                    "leaked": "0",
                    "listsnapshots": "off",
                    "name": "rpool",
                    "readonly": "off",
                    "size": "49.8G",
                    "version": "-"
                }
name:
    description: ZFS pool name
    returned: always
    type: str
    sample: rpool
parsable:
    description: if parsable output should be provided in machine friendly format.
    returned: if 'parsable' is set to True
    type: bool
    sample: True
'''

from collections import defaultdict

from ansible.module_utils.six import iteritems
from ansible.module_utils.basic import AnsibleModule


class ZPoolFacts(object):
    def __init__(self, module):

        self.module = module

        self.name = module.params['name']
        self.parsable = module.params['parsable']
        self.properties = module.params['properties']

        self._pools = defaultdict(dict)
        self.facts = []

    def pool_exists(self):
        cmd = [self.module.get_bin_path('zpool')]

        cmd.append('list')
        cmd.append(self.name)

        (rc, out, err) = self.module.run_command(cmd)

        if rc == 0:
            return True
        else:
            return False

    def get_facts(self):
        cmd = [self.module.get_bin_path('zpool')]

        cmd.append('get')
        cmd.append('-H')
        if self.parsable:
            cmd.append('-p')
        cmd.append('-o')
        cmd.append('name,property,value')
        cmd.append(self.properties)
        if self.name:
            cmd.append(self.name)

        (rc, out, err) = self.module.run_command(cmd)

        if rc == 0:
            for line in out.splitlines():
                pool, property, value = line.split('\t')

                self._pools[pool].update({property: value})

            for k, v in iteritems(self._pools):
                v.update({'name': k})
                self.facts.append(v)

            return {'ansible_zfs_pools': self.facts}
        else:
            self.module.fail_json(msg='Error while trying to get facts about ZFS pool: %s' % self.name,
                                  stderr=err,
                                  rc=rc)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=False, aliases=['pool', 'zpool'], type='str'),
            parsable=dict(required=False, default=False, type='bool'),
            properties=dict(required=False, default='all', type='str'),
        ),
        supports_check_mode=True
    )

    zpool_facts = ZPoolFacts(module)

    result = {}
    result['changed'] = False
    result['name'] = zpool_facts.name

    if zpool_facts.parsable:
        result['parsable'] = zpool_facts.parsable

    if zpool_facts.name is not None:
        if zpool_facts.pool_exists():
            result['ansible_facts'] = zpool_facts.get_facts()
        else:
            module.fail_json(msg='ZFS pool %s does not exist!' % zpool_facts.name)
    else:
        result['ansible_facts'] = zpool_facts.get_facts()

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright 2013 Google Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: gce_pd
short_description: utilize GCE persistent disk resources
description:
    - This module can create and destroy unformatted GCE persistent disks
      U(https://developers.google.com/compute/docs/disks#persistentdisks).
      It also supports attaching and detaching disks from running instances.
      Full install/configuration instructions for the gce* modules can
      be found in the comments of ansible/test/gce_tests.py.
options:
  detach_only:
    description:
      - do not destroy the disk, merely detach it from an instance
    type: bool
  instance_name:
    type: str
    description:
      - instance name if you wish to attach or detach the disk
  mode:
    type: str
    description:
      - GCE mount mode of disk, READ_ONLY (default) or READ_WRITE
    default: "READ_ONLY"
    choices: ["READ_WRITE", "READ_ONLY"]
  name:
    type: str
    description:
      - name of the disk
    required: true
  size_gb:
    type: str
    description:
      - whole integer size of disk (in GB) to create, default is 10 GB
    default: "10"
  image:
    type: str
    description:
      - the source image to use for the disk
  snapshot:
    type: str
    description:
      - the source snapshot to use for the disk
  state:
    type: str
    description:
      - desired state of the persistent disk
      - "Available choices are: C(active), C(present), C(absent), C(deleted)."
    default: "present"
  zone:
    type: str
    description:
      - zone in which to create the disk
    default: "us-central1-b"
  service_account_email:
    type: str
    description:
      - service account email
  pem_file:
    type: path
    description:
      - path to the pem file associated with the service account email
        This option is deprecated. Use 'credentials_file'.
  credentials_file:
    type: path
    description:
      - path to the JSON file associated with the service account email
  project_id:
    type: str
    description:
      - your GCE project ID
  disk_type:
    type: str
    description:
      - Specify a C(pd-standard) disk or C(pd-ssd) for an SSD disk.
    default: "pd-standard"
  delete_on_termination:
    description:
      - If C(yes), deletes the volume when instance is terminated
    type: bool
  image_family:
    type: str
    description:
      - The image family to use to create the instance.
        If I(image) has been used I(image_family) is ignored.
        Cannot specify both I(image) and I(source).
  external_projects:
    type: list
    description:
      - A list of other projects (accessible with the provisioning credentials)
        to be searched for the image.

requirements:
    - "python >= 2.6"
    - "apache-libcloud >= 0.13.3, >= 0.17.0 if using JSON credentials"
author: "Eric Johnson (@erjohnso) <erjohnso@google.com>"
'''

EXAMPLES = '''
- name: Simple attachment action to an existing instance
  local_action:
    module: gce_pd
    instance_name: notlocalhost
    size_gb: 5
    name: pd
'''

try:
    from libcloud.compute.types import Provider
    from libcloud.compute.providers import get_driver
    from libcloud.common.google import GoogleBaseError, QuotaExceededError, ResourceExistsError, ResourceNotFoundError, ResourceInUseError
    _ = Provider.GCE
    HAS_LIBCLOUD = True
except ImportError:
    HAS_LIBCLOUD = False

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.gce import gce_connect, unexpected_error_msg


def main():
    module = AnsibleModule(
        argument_spec=dict(
            delete_on_termination=dict(type='bool'),
            detach_only=dict(type='bool'),
            instance_name=dict(),
            mode=dict(default='READ_ONLY', choices=['READ_WRITE', 'READ_ONLY']),
            name=dict(required=True),
            size_gb=dict(default=10),
            disk_type=dict(default='pd-standard'),
            image=dict(),
            image_family=dict(),
            external_projects=dict(type='list'),
            snapshot=dict(),
            state=dict(default='present'),
            zone=dict(default='us-central1-b'),
            service_account_email=dict(),
            pem_file=dict(type='path'),
            credentials_file=dict(type='path'),
            project_id=dict(),
        )
    )
    if not HAS_LIBCLOUD:
        module.fail_json(msg='libcloud with GCE support (0.17.0+) is required for this module')

    gce = gce_connect(module)

    delete_on_termination = module.params.get('delete_on_termination')
    detach_only = module.params.get('detach_only')
    instance_name = module.params.get('instance_name')
    mode = module.params.get('mode')
    name = module.params.get('name')
    size_gb = module.params.get('size_gb')
    disk_type = module.params.get('disk_type')
    image = module.params.get('image')
    image_family = module.params.get('image_family')
    external_projects = module.params.get('external_projects')
    snapshot = module.params.get('snapshot')
    state = module.params.get('state')
    zone = module.params.get('zone')

    if delete_on_termination and not instance_name:
        module.fail_json(
            msg='Must specify an instance name when requesting delete on termination',
            changed=False)

    if detach_only and not instance_name:
        module.fail_json(
            msg='Must specify an instance name when detaching a disk',
            changed=False)

    disk = inst = None
    changed = is_attached = False

    json_output = {'name': name, 'zone': zone, 'state': state, 'disk_type': disk_type}
    if detach_only:
        json_output['detach_only'] = True
        json_output['detached_from_instance'] = instance_name

    if instance_name:
        # user wants to attach/detach from an existing instance
        try:
            inst = gce.ex_get_node(instance_name, zone)
            # is the disk attached?
            for d in inst.extra['disks']:
                if d['deviceName'] == name:
                    is_attached = True
                    json_output['attached_mode'] = d['mode']
                    json_output['attached_to_instance'] = inst.name
        except Exception:
            pass

    # find disk if it already exists
    try:
        disk = gce.ex_get_volume(name)
        json_output['size_gb'] = int(disk.size)
    except ResourceNotFoundError:
        pass
    except Exception as e:
        module.fail_json(msg=unexpected_error_msg(e), changed=False)

    # user wants a disk to exist.  If "instance_name" is supplied the user
    # also wants it attached
    if state in ['active', 'present']:

        if not size_gb:
            module.fail_json(msg="Must supply a size_gb", changed=False)
        try:
            size_gb = int(round(float(size_gb)))
            if size_gb < 1:
                raise Exception
        except Exception:
            module.fail_json(msg="Must supply a size_gb larger than 1 GB",
                             changed=False)

        if instance_name and inst is None:
            module.fail_json(msg='Instance %s does not exist in zone %s' % (
                instance_name, zone), changed=False)

        if not disk:
            if image is not None and snapshot is not None:
                module.fail_json(
                    msg='Cannot give both image (%s) and snapshot (%s)' % (
                        image, snapshot), changed=False)
            lc_image = None
            lc_snapshot = None
            if image_family is not None:
                lc_image = gce.ex_get_image_from_family(image_family, ex_project_list=external_projects)
            elif image is not None:
                lc_image = gce.ex_get_image(image, ex_project_list=external_projects)
            elif snapshot is not None:
                lc_snapshot = gce.ex_get_snapshot(snapshot)
            try:
                disk = gce.create_volume(
                    size_gb, name, location=zone, image=lc_image,
                    snapshot=lc_snapshot, ex_disk_type=disk_type)
            except ResourceExistsError:
                pass
            except QuotaExceededError:
                module.fail_json(msg='Requested disk size exceeds quota',
                                 changed=False)
            except Exception as e:
                module.fail_json(msg=unexpected_error_msg(e), changed=False)
            json_output['size_gb'] = size_gb
            if image is not None:
                json_output['image'] = image
            if snapshot is not None:
                json_output['snapshot'] = snapshot
            changed = True
        if inst and not is_attached:
            try:
                gce.attach_volume(inst, disk, device=name, ex_mode=mode,
                                  ex_auto_delete=delete_on_termination)
            except Exception as e:
                module.fail_json(msg=unexpected_error_msg(e), changed=False)
            json_output['attached_to_instance'] = inst.name
            json_output['attached_mode'] = mode
            if delete_on_termination:
                json_output['delete_on_termination'] = True
            changed = True

    # user wants to delete a disk (or perhaps just detach it).
    if state in ['absent', 'deleted'] and disk:

        if inst and is_attached:
            try:
                gce.detach_volume(disk, ex_node=inst)
            except Exception as e:
                module.fail_json(msg=unexpected_error_msg(e), changed=False)
            changed = True
        if not detach_only:
            try:
                gce.destroy_volume(disk)
            except ResourceInUseError as e:
                module.fail_json(msg=str(e.value), changed=False)
            except Exception as e:
                module.fail_json(msg=unexpected_error_msg(e), changed=False)
            changed = True

    json_output['changed'] = changed
    module.exit_json(**json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: ucs_managed_objects
short_description: Configures Managed Objects on Cisco UCS Manager
description:
- Configures Managed Objects on Cisco UCS Manager.
- The Python SDK module, Python class within the module (UCSM Class), and all properties must be directly specified.
- More information on the UCSM Python SDK and how to directly configure Managed Objects is available at L(UCSM Python SDK,http://ucsmsdk.readthedocs.io/).
extends_documentation_fragment: cisco.ucs.ucs
options:
  state:
    description:
    - If C(present), will verify that the Managed Objects are present and will create if needed.
    - If C(absent), will verify that the Managed Objects are absent and will delete if needed.
    choices: [ absent, present ]
    default: present
  objects:
    description:
    - List of managed objects to configure.  Each managed object has suboptions the specify the Python SDK module, class, and properties to configure.
    suboptions:
      module:
        description:
        - Name of the Python SDK module implementing the required class.
        required: yes
      class_name:
        description:
        - Name of the Python class that will be used to configure the Managed Object.
        required: yes
      properties:
        description:
        - List of properties to configure on the Managed Object.  See the UCSM Python SDK for information on properties for each class.
        required: yes
      children:
        description:
        - Optional list of child objects.  Each child has its own module, class, and properties suboptions.
        - The parent_mo_or_dn property for child objects is automatically set as the list of children is configured.
    required: yes
requirements:
- ucsmsdk
author:
- David Soper (@dsoper2)
- CiscoUcs (@CiscoUcs)
version_added: '2.8'
'''

EXAMPLES = r'''
- name: Configure Network Control Policy
  cisco.ucs.ucs_managed_objects:
    hostname: 172.16.143.150
    username: admin
    password: password
    objects:
    - module: ucsmsdk.mometa.nwctrl.NwctrlDefinition
      class: NwctrlDefinition
      properties:
        parent_mo_or_dn: org-root
        cdp: enabled
        descr: ''
        lldp_receive: enabled
        lldp_transmit: enabled
        name: Enable-CDP-LLDP

- name: Remove Network Control Policy
  cisco.ucs.ucs_managed_objects:
    hostname: 172.16.143.150
    username: admin
    password: password
    objects:
    - module: ucsmsdk.mometa.nwctrl.NwctrlDefinition
      class: NwctrlDefinition
      properties:
        parent_mo_or_dn: org-root
        name: Enable-CDP-LLDP
    state: absent

- name: Configure Boot Policy Using JSON objects list with children
  cisco.ucs.ucs_managed_objects:
    hostname: 172.16.143.150
    username: admin
    password: password
    objects:
    - {
          "module": "ucsmsdk.mometa.lsboot.LsbootPolicy",
          "class": "LsbootPolicy",
          "properties": {
              "parent_mo_or_dn": "org-root",
              "name": "Python_SDS",
              "enforce_vnic_name": "yes",
              "boot_mode": "legacy",
              "reboot_on_update": "no"
          },
          "children": [
              {
                  "module": "ucsmsdk.mometa.lsboot.LsbootVirtualMedia",
                  "class": "LsbootVirtualMedia",
                  "properties": {
                      "access": "read-only-local",
                      "lun_id": "0",
                      "order": "2"
                  }
              },
              {
                  "module": "ucsmsdk.mometa.lsboot.LsbootStorage",
                  "class": "LsbootStorage",
                  "properties": {
                      "order": "1"
                  },
                  "children": [
                      {
                          "module": "ucsmsdk.mometa.lsboot.LsbootLocalStorage",
                          "class": "LsbootLocalStorage",
                          "properties": {},
                          "children": [
                              {
                                  "module": "ucsmsdk.mometa.lsboot.LsbootDefaultLocalImage",
                                  "class": "LsbootDefaultLocalImage",
                                  "properties": {
                                      "order": "1"
                                  }
                              }
                          ]
                      }
                  ]
              }
          ]
      }

- name: Remove Boot Policy Using JSON objects list
  cisco.ucs.ucs_managed_objects:
    hostname: 172.16.143.150
    username: admin
    password: password
    objects:
    - {
          "module": "ucsmsdk.mometa.lsboot.LsbootPolicy",
          "class": "LsbootPolicy",
          "properties": {
              "parent_mo_or_dn": "org-root",
              "name": "Python_SDS"
          }
      }
    state: absent


'''

RETURN = r'''
#
'''

try:
    from importlib import import_module
    HAS_IMPORT_MODULE = True
except ImportError:
    HAS_IMPORT_MODULE = False

from copy import deepcopy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.ucs.plugins.module_utils.ucs import UCSModule, ucs_argument_spec


def traverse_objects(module, ucs, managed_object, mo=''):
    props_match = False

    mo_module = import_module(managed_object['module'])
    mo_class = getattr(mo_module, managed_object['class'])

    if not managed_object['properties'].get('parent_mo_or_dn'):
        managed_object['properties']['parent_mo_or_dn'] = mo

    mo = mo_class(**managed_object['properties'])

    existing_mo = ucs.login_handle.query_dn(mo.dn)

    if module.params['state'] == 'absent':
        # mo must exist, but all properties do not have to match
        if existing_mo:
            if not module.check_mode:
                ucs.login_handle.remove_mo(existing_mo)
            ucs.result['changed'] = True
    else:
        if existing_mo:
            # check mo props
            kwargs = dict(managed_object['properties'])
            # remove parent info and passwords because those aren't presented in the actual props
            kwargs.pop('parent_mo_or_dn', None)
            kwargs.pop('pwd', None)
            kwargs.pop('password', None)
            if existing_mo.check_prop_match(**kwargs):
                props_match = True

        if not props_match:
            if not module.check_mode:
                ucs.login_handle.add_mo(mo, modify_present=True)
            ucs.result['changed'] = True

    if managed_object.get('children'):
        for child in managed_object['children']:
            # explicit deep copy of child object since traverse_objects may modify parent mo information
            copy_of_child = deepcopy(child)
            traverse_objects(module, ucs, copy_of_child, mo)


def main():
    object_dict = dict(
        module=dict(type='str', required=True),
        class_name=dict(type='str', aliases=['class'], required=True),
        properties=dict(type='dict', required=True),
        children=dict(type='list'),
    )
    argument_spec = ucs_argument_spec
    argument_spec.update(
        objects=dict(type='list', elements='dict', options=object_dict, required=True),
        state=dict(type='str', choices=['present', 'absent'], default='present'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    if not HAS_IMPORT_MODULE:
        module.fail_json(msg='import_module is required for this module')
    ucs = UCSModule(module)

    # note that all objects specified in the object list report a single result (including a single changed).
    ucs.result['changed'] = False

    for managed_object in module.params['objects']:
        traverse_objects(module, ucs, managed_object)
        # single commit for object and any children
        if not module.check_mode and ucs.result['changed']:
            try:
                ucs.login_handle.commit()
            except Exception as e:
                # generic Exception because UCSM can throw a variety of exceptions
                ucs.result['msg'] = "setup error: %s " % str(e)
                module.fail_json(**ucs.result)

    module.exit_json(**ucs.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

try:
    from botocore.exceptions import BotoCoreError, ClientError
except ImportError:
    pass

from .core import is_boto3_error_code
from .ec2 import AWSRetry


def get_elb(connection, module, elb_name):
    """
    Get an ELB based on name. If not found, return None.

    :param connection: AWS boto3 elbv2 connection
    :param module: Ansible module
    :param elb_name: Name of load balancer to get
    :return: boto3 ELB dict or None if not found
    """
    try:
        return _get_elb(connection, module, elb_name)
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e)


@AWSRetry.jittered_backoff()
def _get_elb(connection, module, elb_name):
    """
    Get an ELB based on name using AWSRetry. If not found, return None.

    :param connection: AWS boto3 elbv2 connection
    :param module: Ansible module
    :param elb_name: Name of load balancer to get
    :return: boto3 ELB dict or None if not found
    """

    try:
        load_balancer_paginator = connection.get_paginator('describe_load_balancers')
        return (load_balancer_paginator.paginate(Names=[elb_name]).build_full_result())['LoadBalancers'][0]
    except is_boto3_error_code('LoadBalancerNotFound'):
        return None


def get_elb_listener(connection, module, elb_arn, listener_port):
    """
    Get an ELB listener based on the port provided. If not found, return None.

    :param connection: AWS boto3 elbv2 connection
    :param module: Ansible module
    :param elb_arn: ARN of the ELB to look at
    :param listener_port: Port of the listener to look for
    :return: boto3 ELB listener dict or None if not found
    """

    try:
        listener_paginator = connection.get_paginator('describe_listeners')
        listeners = (AWSRetry.jittered_backoff()(listener_paginator.paginate)(LoadBalancerArn=elb_arn).build_full_result())['Listeners']
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e)

    l = None

    for listener in listeners:
        if listener['Port'] == listener_port:
            l = listener
            break

    return l


def get_elb_listener_rules(connection, module, listener_arn):
    """
    Get rules for a particular ELB listener using the listener ARN.

    :param connection: AWS boto3 elbv2 connection
    :param module: Ansible module
    :param listener_arn: ARN of the ELB listener
    :return: boto3 ELB rules list
    """

    try:
        return AWSRetry.jittered_backoff()(connection.describe_rules)(ListenerArn=listener_arn)['Rules']
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e)


def convert_tg_name_to_arn(connection, module, tg_name):
    """
    Get ARN of a target group using the target group's name

    :param connection: AWS boto3 elbv2 connection
    :param module: Ansible module
    :param tg_name: Name of the target group
    :return: target group ARN string
    """

    try:
        response = AWSRetry.jittered_backoff()(connection.describe_target_groups)(Names=[tg_name])
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e)

    tg_arn = response['TargetGroups'][0]['TargetGroupArn']

    return tg_arn



#!/usr/bin/python
#
# Copyright (c) 2018 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_resource_info
short_description: Generic facts of Azure resources
description:
    - Obtain facts of any resource using Azure REST API.
    - This module gives access to resources that are not supported via Ansible modules.
    - Refer to U(https://docs.microsoft.com/en-us/rest/api/) regarding details related to specific resource REST API.

options:
    url:
        description:
            - Azure RM Resource URL.
    api_version:
        description:
            - Specific API version to be used.
    provider:
        description:
            - Provider type, should be specified in no URL is given.
    resource_group:
        description:
            - Resource group to be used.
            - Required if URL is not specified.
    resource_type:
        description:
            - Resource type.
    resource_name:
        description:
            - Resource name.
    subresource:
        description:
            - List of subresources.
        suboptions:
            namespace:
                description:
                    - Subresource namespace.
            type:
                description:
                    - Subresource type.
            name:
                description:
                    - Subresource name.

extends_documentation_fragment:
- azure.azcollection.azure


author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
  - name: Get scaleset info
    community.azure.azure_rm_resource_info:
      resource_group: myResourceGroup
      provider: compute
      resource_type: virtualmachinescalesets
      resource_name: myVmss
      api_version: "2017-12-01"

  - name: Query all the resources in the resource group
    community.azure.azure_rm_resource_info:
      resource_group: "{{ resource_group }}"
      resource_type: resources
'''

RETURN = '''
response:
    description:
        - Response specific to resource type.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Id of the Azure resource.
            type: str
            returned: always
            sample: "/subscriptions/xxxx...xxxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/virtualMachines/myVM"
        location:
            description:
                - Resource location.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - Resource name.
            type: str
            returned: always
            sample: myVM
        properties:
            description:
                - Specifies the virtual machine's property.
            type: complex
            returned: always
            contains:
                diagnosticsProfile:
                    description:
                        - Specifies the boot diagnostic settings state.
                    type: complex
                    returned: always
                    contains:
                        bootDiagnostics:
                            description:
                                - A debugging feature, which to view Console Output and Screenshot to diagnose VM status.
                            type: dict
                            returned: always
                            sample: {
                                    "enabled": true,
                                    "storageUri": "https://vxisurgdiag.blob.core.windows.net/"
                                    }
                hardwareProfile:
                    description:
                        - Specifies the hardware settings for the virtual machine.
                    type: dict
                    returned: always
                    sample: {
                            "vmSize": "Standard_D2s_v3"
                            }
                networkProfile:
                    description:
                        - Specifies the network interfaces of the virtual machine.
                    type: complex
                    returned: always
                    contains:
                        networkInterfaces:
                            description:
                                - Describes a network interface reference.
                            type: list
                            returned: always
                            sample:
                                - {
                                  "id": "/subscriptions/xxxx...xxxx/resourceGroups/v-xisuRG/providers/Microsoft.Network/networkInterfaces/myvm441"
                                  }
                osProfile:
                    description:
                        - Specifies the operating system settings for the virtual machine.
                    type: complex
                    returned: always
                    contains:
                        adminUsername:
                            description:
                                - Specifies the name of the administrator account.
                            type: str
                            returned: always
                            sample: azureuser
                        allowExtensionOperations:
                            description:
                                - Specifies whether extension operations should be allowed on the virtual machine.
                                - This may only be set to False when no extensions are present on the virtual machine.
                            type: bool
                            returned: always
                            sample: true
                        computerName:
                            description:
                                - Specifies the host OS name of the virtual machine.
                            type: str
                            returned: always
                            sample: myVM
                        requireGuestProvisionSignale:
                            description:
                                - Specifies the host require guest provision signal or not.
                            type: bool
                            returned: always
                            sample: true
                        secrets:
                            description:
                                - Specifies set of certificates that should be installed onto the virtual machine.
                            type: list
                            returned: always
                            sample: []
                        linuxConfiguration:
                            description:
                                - Specifies the Linux operating system settings on the virtual machine.
                            type: dict
                            returned: when OS type is Linux
                            sample: {
                                    "disablePasswordAuthentication": false,
                                    "provisionVMAgent": true
                                     }
                provisioningState:
                    description:
                        - The provisioning state.
                    type: str
                    returned: always
                    sample: Succeeded
                vmID:
                    description:
                        - Specifies the VM unique ID which is a 128-bits identifier that is encoded and stored in all Azure laaS VMs SMBIOS.
                        - It can be read using platform BIOS commands.
                    type: str
                    returned: always
                    sample: "eb86d9bb-6725-4787-a487-2e497d5b340c"
                storageProfile:
                    description:
                        - Specifies the storage account type for the managed disk.
                    type: complex
                    returned: always
                    contains:
                        dataDisks:
                            description:
                                - Specifies the parameters that are used to add a data disk to virtual machine.
                            type: list
                            returned: always
                            sample:
                                - {
                                  "caching": "None",
                                  "createOption": "Attach",
                                  "diskSizeGB": 1023,
                                  "lun": 2,
                                  "managedDisk": {
                                                "id": "/subscriptions/xxxx....xxxx/resourceGroups/V-XISURG/providers/Microsoft.Compute/disks/testdisk2",
                                                 "storageAccountType": "StandardSSD_LRS"
                                                },
                                  "name": "testdisk2"
                                   }
                                - {
                                  "caching": "None",
                                  "createOption": "Attach",
                                  "diskSizeGB": 1023,
                                  "lun": 1,
                                  "managedDisk": {
                                                "id": "/subscriptions/xxxx...xxxx/resourceGroups/V-XISURG/providers/Microsoft.Compute/disks/testdisk3",
                                                "storageAccountType": "StandardSSD_LRS"
                                                },
                                  "name": "testdisk3"
                                  }

                        imageReference:
                            description:
                                - Specifies information about the image to use.
                            type: dict
                            returned: always
                            sample: {
                                   "offer": "UbuntuServer",
                                   "publisher": "Canonical",
                                   "sku": "18.04-LTS",
                                   "version": "latest"
                                   }
                        osDisk:
                            description:
                                - Specifies information about the operating system disk used by the virtual machine.
                            type: dict
                            returned: always
                            sample: {
                                   "caching": "ReadWrite",
                                   "createOption": "FromImage",
                                   "diskSizeGB": 30,
                                   "managedDisk": {
                                                  "id": "/subscriptions/xxx...xxxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/disks/myVM_disk1_xxx",
                                                  "storageAccountType": "Premium_LRS"
                                                   },
                                   "name": "myVM_disk1_xxx",
                                   "osType": "Linux"
                                   }
        type:
            description:
                - The type of identity used for the virtual machine.
            type: str
            returned: always
            sample: "Microsoft.Compute/virtualMachines"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient

try:
    from msrestazure.azure_exceptions import CloudError
    from msrest.service_client import ServiceClient
    from msrestazure.tools import resource_id, is_valid_resource_id
    import json

except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMResourceInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            url=dict(
                type='str'
            ),
            provider=dict(
                type='str'
            ),
            resource_group=dict(
                type='str'
            ),
            resource_type=dict(
                type='str'
            ),
            resource_name=dict(
                type='str'
            ),
            subresource=dict(
                type='list',
                default=[]
            ),
            api_version=dict(
                type='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            response=[]
        )
        self.mgmt_client = None
        self.url = None
        self.api_version = None
        self.provider = None
        self.resource_group = None
        self.resource_type = None
        self.resource_name = None
        self.subresource = []
        super(AzureRMResourceInfo, self).__init__(self.module_arg_spec, supports_tags=False)

    def exec_module(self, **kwargs):
        is_old_facts = self.module._name == 'azure_rm_resource_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_resource_facts' module has been renamed to 'azure_rm_resource_info'", version='2.13')

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])
        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if self.url is None:
            orphan = None
            rargs = dict()
            rargs['subscription'] = self.subscription_id
            rargs['resource_group'] = self.resource_group
            if not (self.provider is None or self.provider.lower().startswith('.microsoft')):
                rargs['namespace'] = "Microsoft." + self.provider
            else:
                rargs['namespace'] = self.provider

            if self.resource_type is not None and self.resource_name is not None:
                rargs['type'] = self.resource_type
                rargs['name'] = self.resource_name
                for i in range(len(self.subresource)):
                    resource_ns = self.subresource[i].get('namespace', None)
                    resource_type = self.subresource[i].get('type', None)
                    resource_name = self.subresource[i].get('name', None)
                    if resource_type is not None and resource_name is not None:
                        rargs['child_namespace_' + str(i + 1)] = resource_ns
                        rargs['child_type_' + str(i + 1)] = resource_type
                        rargs['child_name_' + str(i + 1)] = resource_name
                    else:
                        orphan = resource_type
            else:
                orphan = self.resource_type

            self.url = resource_id(**rargs)

            if orphan is not None:
                self.url += '/' + orphan

        # if api_version was not specified, get latest one
        if not self.api_version:
            try:
                # extract provider and resource type
                if "/providers/" in self.url:
                    provider = self.url.split("/providers/")[1].split("/")[0]
                    resourceType = self.url.split(provider + "/")[1].split("/")[0]
                    url = "/subscriptions/" + self.subscription_id + "/providers/" + provider
                    api_versions = json.loads(self.mgmt_client.query(url, "GET", {'api-version': '2015-01-01'}, None, None, [200], 0, 0).text)
                    for rt in api_versions['resourceTypes']:
                        if rt['resourceType'].lower() == resourceType.lower():
                            self.api_version = rt['apiVersions'][0]
                            break
                else:
                    # if there's no provider in API version, assume Microsoft.Resources
                    self.api_version = '2018-05-01'
                if not self.api_version:
                    self.fail("Couldn't find api version for {0}/{1}".format(provider, resourceType))
            except Exception as exc:
                self.fail("Failed to obtain API version: {0}".format(str(exc)))

        self.results['url'] = self.url

        query_parameters = {}
        query_parameters['api-version'] = self.api_version

        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        skiptoken = None

        while True:
            if skiptoken:
                query_parameters['skiptoken'] = skiptoken
            response = self.mgmt_client.query(self.url, "GET", query_parameters, header_parameters, None, [200, 404], 0, 0)
            try:
                response = json.loads(response.text)
                if isinstance(response, dict):
                    if response.get('value'):
                        self.results['response'] = self.results['response'] + response['value']
                        skiptoken = response.get('nextLink')
                    else:
                        self.results['response'] = self.results['response'] + [response]
            except Exception as e:
                self.fail('Failed to parse response: ' + str(e))
            if not skiptoken:
                break
        return self.results


def main():
    AzureRMResourceInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, John Westcott IV <john.westcott.iv@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: tower_import
author: "John Westcott (@john-westcott-iv)"
version_added: "3.7"
short_description: import resources into Ansible Tower.
description:
    - Import assets into Ansible Tower. See
      U(https://www.ansible.com/tower) for an overview.
options:
    assets:
      description:
        - The assets to import.
        - This can be the output of tower_export or loaded from a file
      required: True
      type: dict
requirements:
  - "awxkit >= 9.3.0"
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Export all assets
  tower_export:
    all: True
  registeR: export_output

- name: Import all tower assets from our export
  tower_import:
    assets: "{{ export_output.assets }}"

- name: Load data from a json file created by a command like awx export --organization Default
  tower_import:
    assets: "{{ lookup('file', 'org.json') | from_json() }}"
'''

from ..module_utils.tower_awxkit import TowerAWXKitModule

# These two lines are not needed if awxkit changes to do progamatic notifications on issues
from ansible.module_utils.six.moves import StringIO
import logging

# In this module we don't use EXPORTABLE_RESOURCES, we just want to validate that our installed awxkit has import/export
try:
    from awxkit.api.pages.api import EXPORTABLE_RESOURCES
    HAS_EXPORTABLE_RESOURCES = True
except ImportError:
    HAS_EXPORTABLE_RESOURCES = False


def main():
    argument_spec = dict(
        assets=dict(type='dict', required=True)
    )

    module = TowerAWXKitModule(argument_spec=argument_spec, supports_check_mode=False)

    assets = module.params.get('assets')

    if not HAS_EXPORTABLE_RESOURCES:
        module.fail_json(msg="Your version of awxkit does not appear to have import/export")

    # Currently the import process does not return anything on error
    # It simply just logs to pythons logger
    # Setup a log gobbler to get error messages from import_assets
    logger = logging.getLogger('awxkit.api.pages.api')
    logger.setLevel(logging.WARNING)
    log_capture_string = StringIO()
    ch = logging.StreamHandler(log_capture_string)
    ch.setLevel(logging.WARNING)
    logger.addHandler(ch)
    log_contents = ''

    # Run the import process
    try:
        module.json_output['changed'] = module.get_api_v2_object().import_assets(assets)
    except Exception as e:
        module.fail_json(msg="Failed to import assets {0}".format(e))
    finally:
        # Finally consume the logs incase there were any errors and die if there were
        log_contents = log_capture_string.getvalue()
        log_capture_string.close()
        if log_contents != '':
            module.fail_json(msg=log_contents)

    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2013, Balazs Pocze <banyek@gawker.com>
# Certain parts are taken from Mark Theunissen's mysqldb module
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_variables

short_description: Manage MySQL global variables
description:
- Query / Set MySQL variables.
author:
- Balazs Pocze (@banyek)
options:
  variable:
    description:
    - Variable name to operate.
    type: str
    required: yes
  value:
    description:
    - If set, then sets variable value to this.
    type: str
  mode:
    description:
    - C(global) assigns C(value) to a global system variable which will be changed at runtime
      but won't persist across server restarts.
    - C(persist) assigns C(value) to a global system variable and persists it to
      the mysqld-auto.cnf option file in the data directory
      (the variable will survive service restarts).
    - C(persist_only) persists C(value) to the mysqld-auto.cnf option file in the data directory
      but without setting the global variable runtime value
      (the value will be changed after the next service restart).
    - Supported by MySQL 8.0 or later.
    - For more information see U(https://dev.mysql.com/doc/refman/8.0/en/set-variable.html).
    type: str
    choices: ['global', 'persist', 'persist_only']
    default: global
    version_added: '0.1.0'

notes:
- Does not support C(check_mode).

seealso:
- module: community.mysql.mysql_info
- name: MySQL SET command reference
  description: Complete reference of the MySQL SET command documentation.
  link: https://dev.mysql.com/doc/refman/8.0/en/set-statement.html

extends_documentation_fragment:
- community.mysql.mysql
'''

EXAMPLES = r'''
- name: Check for sync_binlog setting
  community.mysql.mysql_variables:
    variable: sync_binlog

- name: Set read_only variable to 1 persistently
  community.mysql.mysql_variables:
    variable: read_only
    value: 1
    mode: persist
'''

RETURN = r'''
queries:
  description: List of executed queries which modified DB's state.
  returned: if executed
  type: list
  sample: ["SET GLOBAL `read_only` = 1"]
  version_added: '0.1.0'
'''

import os
import warnings
from re import match

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.mysql.plugins.module_utils.database import SQLParseError, mysql_quote_identifier
from ansible_collections.community.mysql.plugins.module_utils.mysql import mysql_connect, mysql_driver, mysql_driver_fail_msg, mysql_common_argument_spec
from ansible.module_utils._text import to_native

executed_queries = []


def check_mysqld_auto(module, cursor, mysqlvar):
    """Check variable's value in mysqld-auto.cnf."""
    query = ("SELECT VARIABLE_VALUE "
             "FROM performance_schema.persisted_variables "
             "WHERE VARIABLE_NAME = %s")
    try:
        cursor.execute(query, (mysqlvar,))
        res = cursor.fetchone()
    except Exception as e:
        if "Table 'performance_schema.persisted_variables' doesn't exist" in str(e):
            module.fail_json(msg='Server version must be 8.0 or greater.')

    if res:
        return res[0]
    else:
        return None


def typedvalue(value):
    """
    Convert value to number whenever possible, return same value
    otherwise.

    >>> typedvalue('3')
    3
    >>> typedvalue('3.0')
    3.0
    >>> typedvalue('foobar')
    'foobar'

    """
    try:
        return int(value)
    except ValueError:
        pass

    try:
        return float(value)
    except ValueError:
        pass

    return value


def getvariable(cursor, mysqlvar):
    cursor.execute("SHOW VARIABLES WHERE Variable_name = %s", (mysqlvar,))
    mysqlvar_val = cursor.fetchall()
    if len(mysqlvar_val) == 1:
        return mysqlvar_val[0][1]
    else:
        return None


def setvariable(cursor, mysqlvar, value, mode='global'):
    """ Set a global mysql variable to a given value

    The DB driver will handle quoting of the given value based on its
    type, thus numeric strings like '3.0' or '8' are illegal, they
    should be passed as numeric literals.

    """
    if mode == 'persist':
        query = "SET PERSIST %s = " % mysql_quote_identifier(mysqlvar, 'vars')
    elif mode == 'global':
        query = "SET GLOBAL %s = " % mysql_quote_identifier(mysqlvar, 'vars')
    elif mode == 'persist_only':
        query = "SET PERSIST_ONLY %s = " % mysql_quote_identifier(mysqlvar, 'vars')

    try:
        cursor.execute(query + "%s", (value,))
        executed_queries.append(query + "%s" % value)
        cursor.fetchall()
        result = True
    except Exception as e:
        result = to_native(e)

    return result


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        variable=dict(type='str'),
        value=dict(type='str'),
        mode=dict(type='str', choices=['global', 'persist', 'persist_only'], default='global'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec
    )
    user = module.params["login_user"]
    password = module.params["login_password"]
    connect_timeout = module.params['connect_timeout']
    ssl_cert = module.params["client_cert"]
    ssl_key = module.params["client_key"]
    ssl_ca = module.params["ca_cert"]
    check_hostname = module.params["check_hostname"]
    config_file = module.params['config_file']
    db = 'mysql'

    mysqlvar = module.params["variable"]
    value = module.params["value"]
    mode = module.params["mode"]

    if mysqlvar is None:
        module.fail_json(msg="Cannot run without variable to operate with")
    if match('^[0-9a-z_.]+$', mysqlvar) is None:
        module.fail_json(msg="invalid variable name \"%s\"" % mysqlvar)
    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)
    else:
        warnings.filterwarnings('error', category=mysql_driver.Warning)

    try:
        cursor, db_conn = mysql_connect(module, user, password, config_file, ssl_cert, ssl_key, ssl_ca, db,
                                        connect_timeout=connect_timeout, check_hostname=check_hostname)
    except Exception as e:
        if os.path.exists(config_file):
            module.fail_json(msg=("unable to connect to database, check login_user and "
                                  "login_password are correct or %s has the credentials. "
                                  "Exception message: %s" % (config_file, to_native(e))))
        else:
            module.fail_json(msg="unable to find %s. Exception message: %s" % (config_file, to_native(e)))

    mysqlvar_val = None
    var_in_mysqld_auto_cnf = None

    mysqlvar_val = getvariable(cursor, mysqlvar)
    if mysqlvar_val is None:
        module.fail_json(msg="Variable not available \"%s\"" % mysqlvar, changed=False)

    if value is None:
        module.exit_json(msg=mysqlvar_val)

    if mode in ('persist', 'persist_only'):
        var_in_mysqld_auto_cnf = check_mysqld_auto(module, cursor, mysqlvar)

        if mode == 'persist_only':
            if var_in_mysqld_auto_cnf is None:
                mysqlvar_val = False
            else:
                mysqlvar_val = var_in_mysqld_auto_cnf

    # Type values before using them
    value_wanted = typedvalue(value)
    value_actual = typedvalue(mysqlvar_val)
    value_in_auto_cnf = None
    if var_in_mysqld_auto_cnf is not None:
        value_in_auto_cnf = typedvalue(var_in_mysqld_auto_cnf)

    if value_wanted == value_actual and mode in ('global', 'persist'):
        if mode == 'persist' and value_wanted == value_in_auto_cnf:
            module.exit_json(msg="Variable is already set to requested value globally"
                                 "and stored into mysqld-auto.cnf file.", changed=False)

        elif mode == 'global':
            module.exit_json(msg="Variable is already set to requested value.", changed=False)

    if mode == 'persist_only' and value_wanted == value_in_auto_cnf:
        module.exit_json(msg="Variable is already stored into mysqld-auto.cnf "
                             "with requested value.", changed=False)

    try:
        result = setvariable(cursor, mysqlvar, value_wanted, mode)
    except SQLParseError as e:
        result = to_native(e)

    if result is True:
        module.exit_json(msg="Variable change succeeded prev_value=%s" % value_actual,
                         changed=True, queries=executed_queries)
    else:
        module.fail_json(msg=result, changed=False)


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################
"""
The arg spec for the ios_acls module
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class AclsArgs(object):
    """The arg spec for the ios_acls module
    """

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "elements": "dict",
            "options": {
                "afi": {
                    "required": True,
                    "choices": ["ipv4", "ipv6"],
                    "type": "str",
                },
                "acls": {
                    "elements": "dict",
                    "type": "list",
                    "options": {
                        "name": {"required": True, "type": "str"},
                        "acl_type": {
                            "choices": ["extended", "standard"],
                            "type": "str",
                        },
                        "aces": {
                            "elements": "dict",
                            "type": "list",
                            "options": {
                                "grant": {
                                    "choices": ["permit", "deny"],
                                    "type": "str",
                                },
                                "sequence": {"type": "int"},
                                "evaluate": {"type": "str"},
                                "source": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["address", "any", "host"],
                                        ["wildcard_bits", "any", "host"],
                                    ],
                                    "options": {
                                        "address": {"type": "str"},
                                        "wildcard_bits": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "port_protocol": {
                                            "type": "dict",
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "start": {
                                                            "type": "int"
                                                        },
                                                        "end": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "destination": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["address", "any", "host"],
                                        ["wildcard_bits", "any", "host"],
                                    ],
                                    "options": {
                                        "address": {"type": "str"},
                                        "wildcard_bits": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "port_protocol": {
                                            "type": "dict",
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "start": {
                                                            "type": "int"
                                                        },
                                                        "end": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "protocol": {"type": "str"},
                                "protocol_options": {
                                    "type": "dict",
                                    "options": {
                                        "protocol_number": {"type": "int"},
                                        "ahp": {"type": "bool"},
                                        "eigrp": {"type": "bool"},
                                        "esp": {"type": "bool"},
                                        "gre": {"type": "bool"},
                                        "hbh": {"type": "bool"},
                                        "icmp": {
                                            "type": "dict",
                                            "options": {
                                                "administratively_prohibited": {
                                                    "type": "bool"
                                                },
                                                "alternate_address": {
                                                    "type": "bool"
                                                },
                                                "conversion_error": {
                                                    "type": "bool"
                                                },
                                                "dod_host_prohibited": {
                                                    "type": "bool"
                                                },
                                                "dod_net_prohibited": {
                                                    "type": "bool"
                                                },
                                                "echo": {"type": "bool"},
                                                "echo_reply": {"type": "bool"},
                                                "general_parameter_problem": {
                                                    "type": "bool"
                                                },
                                                "host_isolated": {
                                                    "type": "bool"
                                                },
                                                "host_precedence_unreachable": {
                                                    "type": "bool"
                                                },
                                                "host_redirect": {
                                                    "type": "bool"
                                                },
                                                "host_tos_redirect": {
                                                    "type": "bool"
                                                },
                                                "host_tos_unreachable": {
                                                    "type": "bool"
                                                },
                                                "host_unknown": {
                                                    "type": "bool"
                                                },
                                                "host_unreachable": {
                                                    "type": "bool"
                                                },
                                                "information_reply": {
                                                    "type": "bool"
                                                },
                                                "information_request": {
                                                    "type": "bool"
                                                },
                                                "mask_reply": {"type": "bool"},
                                                "mask_request": {
                                                    "type": "bool"
                                                },
                                                "mobile_redirect": {
                                                    "type": "bool"
                                                },
                                                "net_redirect": {
                                                    "type": "bool"
                                                },
                                                "net_tos_redirect": {
                                                    "type": "bool"
                                                },
                                                "net_tos_unreachable": {
                                                    "type": "bool"
                                                },
                                                "net_unreachable": {
                                                    "type": "bool"
                                                },
                                                "network_unknown": {
                                                    "type": "bool"
                                                },
                                                "no_room_for_option": {
                                                    "type": "bool"
                                                },
                                                "option_missing": {
                                                    "type": "bool"
                                                },
                                                "packet_too_big": {
                                                    "type": "bool"
                                                },
                                                "parameter_problem": {
                                                    "type": "bool"
                                                },
                                                "port_unreachable": {
                                                    "type": "bool"
                                                },
                                                "precedence_unreachable": {
                                                    "type": "bool"
                                                },
                                                "protocol_unreachable": {
                                                    "type": "bool"
                                                },
                                                "reassembly_timeout": {
                                                    "type": "bool"
                                                },
                                                "redirect": {"type": "bool"},
                                                "router_advertisement": {
                                                    "type": "bool"
                                                },
                                                "router_solicitation": {
                                                    "type": "bool"
                                                },
                                                "source_quench": {
                                                    "type": "bool"
                                                },
                                                "source_route_failed": {
                                                    "type": "bool"
                                                },
                                                "time_exceeded": {
                                                    "type": "bool"
                                                },
                                                "timestamp_reply": {
                                                    "type": "bool"
                                                },
                                                "timestamp_request": {
                                                    "type": "bool"
                                                },
                                                "traceroute": {"type": "bool"},
                                                "ttl_exceeded": {
                                                    "type": "bool"
                                                },
                                                "unreachable": {
                                                    "type": "bool"
                                                },
                                            },
                                        },
                                        "igmp": {
                                            "type": "dict",
                                            "options": {
                                                "dvmrp": {"type": "bool"},
                                                "host_query": {"type": "bool"},
                                                "mtrace_resp": {
                                                    "type": "bool"
                                                },
                                                "mtrace_route": {
                                                    "type": "bool"
                                                },
                                                "pim": {"type": "bool"},
                                                "trace": {"type": "bool"},
                                                "v1host_report": {
                                                    "type": "bool"
                                                },
                                                "v2host_report": {
                                                    "type": "bool"
                                                },
                                                "v2leave_group": {
                                                    "type": "bool"
                                                },
                                                "v3host_report": {
                                                    "type": "bool"
                                                },
                                            },
                                        },
                                        "ip": {"type": "bool"},
                                        "ipv6": {"type": "bool"},
                                        "ipinip": {"type": "bool"},
                                        "nos": {"type": "bool"},
                                        "ospf": {"type": "bool"},
                                        "pcp": {"type": "bool"},
                                        "pim": {"type": "bool"},
                                        "sctp": {"type": "bool"},
                                        "tcp": {
                                            "options": {
                                                "ack": {"type": "bool"},
                                                "established": {
                                                    "type": "bool"
                                                },
                                                "fin": {"type": "bool"},
                                                "psh": {"type": "bool"},
                                                "rst": {"type": "bool"},
                                                "syn": {"type": "bool"},
                                                "urg": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "udp": {"type": "bool"},
                                    },
                                },
                                "dscp": {"type": "str"},
                                "fragments": {"type": "str"},
                                "log": {"type": "str"},
                                "log_input": {"type": "str"},
                                "option": {
                                    "type": "dict",
                                    "options": {
                                        "add_ext": {"type": "bool"},
                                        "any_options": {"type": "bool"},
                                        "com_security": {"type": "bool"},
                                        "dps": {"type": "bool"},
                                        "encode": {"type": "bool"},
                                        "eool": {"type": "bool"},
                                        "ext_ip": {"type": "bool"},
                                        "ext_security": {"type": "bool"},
                                        "finn": {"type": "bool"},
                                        "imitd": {"type": "bool"},
                                        "lsr": {"type": "bool"},
                                        "mtup": {"type": "bool"},
                                        "mtur": {"type": "bool"},
                                        "no_op": {"type": "bool"},
                                        "nsapa": {"type": "bool"},
                                        "record_route": {"type": "bool"},
                                        "router_alert": {"type": "bool"},
                                        "sdb": {"type": "bool"},
                                        "security": {"type": "bool"},
                                        "ssr": {"type": "bool"},
                                        "stream_id": {"type": "bool"},
                                        "timestamp": {"type": "bool"},
                                        "traceroute": {"type": "bool"},
                                        "ump": {"type": "bool"},
                                        "visa": {"type": "bool"},
                                        "zsu": {"type": "bool"},
                                    },
                                },
                                "precedence": {"type": "int"},
                                "time_range": {"type": "str"},
                                "tos": {
                                    "type": "dict",
                                    "options": {
                                        "service_value": {"type": "int"},
                                        "max_reliability": {"type": "bool"},
                                        "max_throughput": {"type": "bool"},
                                        "min_delay": {"type": "bool"},
                                        "min_monetary_cost": {"type": "bool"},
                                        "normal": {"type": "bool"},
                                    },
                                },
                                "ttl": {
                                    "type": "dict",
                                    "options": {
                                        "eq": {"type": "int"},
                                        "gt": {"type": "int"},
                                        "lt": {"type": "int"},
                                        "neq": {"type": "int"},
                                        "range": {
                                            "type": "dict",
                                            "options": {
                                                "start": {"type": "int"},
                                                "end": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
            "type": "list",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }



#!/usr/bin/python
# Copyright 2017 Google Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
module: gce_eip
short_description: Create or Destroy Global or Regional External IP addresses.
description:
    - Create (reserve) or Destroy (release) Regional or Global IP Addresses. See
      U(https://cloud.google.com/compute/docs/configure-instance-ip-addresses#reserve_new_static) for more on reserving static addresses.
requirements:
  - "python >= 2.6"
  - "apache-libcloud >= 0.19.0"
notes:
  - Global addresses can only be used with Global Forwarding Rules.
author:
  - "Tom Melendez (@supertom) <tom@supertom.com>"
options:
  name:
    type: str
    description:
       - Name of Address.
    required: true
  region:
    type: str
    description:
       - Region to create the address in. Set to 'global' to create a global address.
    required: true
  state:
    type: str
    description: The state the address should be in. C(present) or C(absent) are the only valid options.
    default: present
    required: false
    choices: [present, absent]
  project_id:
    type: str
    description:
      - The Google Cloud Platform project ID to use.
  pem_file:
    type: path
    description:
      - The path to the PEM file associated with the service account email.
      - This option is deprecated and may be removed in a future release. Use I(credentials_file) instead.
  credentials_file:
    type: path
    description:
      - The path to the JSON file associated with the service account email.
  service_account_email:
    type: str
    description:
      - service account email
  service_account_permissions:
    type: list
    description:
      - service account permissions
'''

EXAMPLES = '''
- name: Create a Global external IP address
  community.google.gce_eip:
    service_account_email: "{{ service_account_email }}"
    credentials_file: "{{ credentials_file }}"
    project_id: "{{ project_id }}"
    name: my-global-ip
    region: global
    state: present

- name: Create a Regional external IP address
  community.google.gce_eip:
    service_account_email: "{{ service_account_email }}"
    credentials_file: "{{ credentials_file }}"
    project_id: "{{ project_id }}"
    name: my-global-ip
    region: us-east1
    state: present
'''

RETURN = '''
address:
    description: IP address being operated on
    returned: always
    type: str
    sample: "35.186.222.233"
name:
    description: name of the address being operated on
    returned: always
    type: str
    sample: "my-address"
region:
    description: Which region an address belongs.
    returned: always
    type: str
    sample: "global"
'''

USER_AGENT_VERSION = 'v1'
USER_AGENT_PRODUCT = 'Ansible-gce_eip'

try:
    from ast import literal_eval
    HAS_PYTHON26 = True
except ImportError:
    HAS_PYTHON26 = False

try:
    import libcloud
    from libcloud.compute.types import Provider
    from libcloud.compute.providers import get_driver
    from libcloud.common.google import GoogleBaseError, QuotaExceededError, \
        ResourceExistsError, ResourceInUseError, ResourceNotFoundError
    from libcloud.compute.drivers.gce import GCEAddress
    _ = Provider.GCE
    HAS_LIBCLOUD = True
except ImportError:
    HAS_LIBCLOUD = False

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.google.plugins.module_utils.gcp import gcp_connect


def get_address(gce, name, region):
    """
    Get an Address from GCE.

    :param gce: An initialized GCE driver object.
    :type gce:  :class: `GCENodeDriver`

    :param name: Name of the Address.
    :type name:  ``str``

    :return: A GCEAddress object or None.
    :rtype: :class: `GCEAddress` or None
    """
    try:
        return gce.ex_get_address(name=name, region=region)

    except ResourceNotFoundError:
        return None


def create_address(gce, params):
    """
    Create a new Address.

    :param gce: An initialized GCE driver object.
    :type gce:  :class: `GCENodeDriver`

    :param params: Dictionary of parameters needed by the module.
    :type params:  ``dict``

    :return: Tuple with changed status and address.
    :rtype: tuple in the format of (bool, str)
    """
    changed = False
    return_data = []

    address = gce.ex_create_address(
        name=params['name'], region=params['region'])

    if address:
        changed = True
        return_data = address.address

    return (changed, return_data)


def delete_address(address):
    """
    Delete an Address.

    :param gce: An initialized GCE driver object.
    :type gce:  :class: `GCENodeDriver`

    :param params: Dictionary of parameters needed by the module.
    :type params:  ``dict``

    :return: Tuple with changed status and address.
    :rtype: tuple in the format of (bool, str)
    """
    changed = False
    return_data = []
    if address.destroy():
        changed = True
        return_data = address.address
    return (changed, return_data)


def main():
    module = AnsibleModule(argument_spec=dict(
        name=dict(required=True),
        state=dict(choices=['absent', 'present'], default='present'),
        region=dict(required=True),
        service_account_email=dict(),
        service_account_permissions=dict(type='list'),
        pem_file=dict(type='path'),
        credentials_file=dict(type='path'),
        project_id=dict(), ), )

    if not HAS_PYTHON26:
        module.fail_json(
            msg="GCE module requires python's 'ast' module, python v2.6+")
    if not HAS_LIBCLOUD:
        module.fail_json(
            msg='libcloud with GCE support (+0.19) required for this module.')

    gce = gcp_connect(module, Provider.GCE, get_driver,
                      USER_AGENT_PRODUCT, USER_AGENT_VERSION)

    params = {}
    params['state'] = module.params.get('state')
    params['name'] = module.params.get('name')
    params['region'] = module.params.get('region')

    changed = False
    json_output = {'state': params['state']}
    address = get_address(gce, params['name'], region=params['region'])

    if params['state'] == 'absent':
        if not address:
            # Doesn't exist in GCE, and state==absent.
            changed = False
            module.fail_json(
                msg="Cannot delete unknown address: %s" %
                (params['name']))
        else:
            # Delete
            (changed, json_output['address']) = delete_address(address)
    else:
        if not address:
            # Create
            (changed, json_output['address']) = create_address(gce,
                                                               params)
        else:
            changed = False
            json_output['address'] = address.address

    json_output['changed'] = changed
    json_output.update(params)
    module.exit_json(**json_output)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The arg spec for the eos_ospfv3 module
"""


class Ospfv3Args(object):  # pylint: disable=R0903
    """The arg spec for the eos_ospfv3 module
    """

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "running_config": {"type": "str"},
        "state": {
            "default": "merged",
            "type": "str",
            "choices": [
                "deleted",
                "merged",
                "overridden",
                "replaced",
                "gathered",
                "rendered",
                "parsed",
            ],
        },
        "config": {
            "type": "dict",
            "options": {
                "processes": {
                    "elements": "dict",
                    "type": "list",
                    "options": {
                        "router_id": {"type": "str"},
                        "shutdown": {"type": "bool"},
                        "fips_restrictions": {"type": "bool"},
                        "graceful_restart_helper": {"type": "bool"},
                        "adjacency": {
                            "type": "dict",
                            "options": {
                                "exchange_start": {
                                    "type": "dict",
                                    "options": {"threshold": {"type": "int"}},
                                }
                            },
                        },
                        "max_metric": {
                            "type": "dict",
                            "options": {
                                "router_lsa": {
                                    "type": "dict",
                                    "options": {
                                        "external_lsa": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "max_metric_value": {
                                                    "type": "int"
                                                },
                                            },
                                        },
                                        "summary_lsa": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "max_metric_value": {
                                                    "type": "int"
                                                },
                                            },
                                        },
                                        "set": {"type": "bool"},
                                        "on_startup": {
                                            "type": "dict",
                                            "options": {
                                                "wait_for_bgp": {
                                                    "type": "bool"
                                                },
                                                "wait_period": {"type": "int"},
                                            },
                                        },
                                        "include_stub": {"type": "bool"},
                                    },
                                }
                            },
                        },
                        "log_adjacency_changes": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "detail": {"type": "bool"},
                            },
                        },
                        "graceful_restart": {
                            "type": "dict",
                            "options": {
                                "grace_period": {"type": "int"},
                                "set": {"type": "bool"},
                            },
                        },
                        "timers": {
                            "type": "dict",
                            "options": {
                                "throttle": {
                                    "type": "dict",
                                    "options": {
                                        "max": {"type": "int"},
                                        "initial": {"type": "int"},
                                        "min": {"type": "int"},
                                        "spf": {"type": "bool"},
                                        "lsa": {"type": "bool"},
                                    },
                                },
                                "out_delay": {"type": "int"},
                                "pacing": {"type": "int"},
                                "lsa": {"type": "int"},
                            },
                        },
                        "vrf": {"type": "str"},
                        "auto_cost": {
                            "type": "dict",
                            "options": {
                                "reference_bandwidth": {"type": "int"}
                            },
                        },
                        "passive_interface": {"type": "bool"},
                        "bfd": {
                            "type": "dict",
                            "options": {"all_interfaces": {"type": "bool"}},
                        },
                        "areas": {
                            "elements": "dict",
                            "type": "list",
                            "options": {
                                "area_id": {"type": "str"},
                                "encryption": {
                                    "type": "dict",
                                    "options": {
                                        "hidden_key": {"type": "bool"},
                                        "key": {"type": "str"},
                                        "algorithm": {
                                            "type": "str",
                                            "choices": ["sha1", "md5"],
                                        },
                                        "encrypt_key": {"type": "bool"},
                                        "encryption": {
                                            "type": "str",
                                            "choices": [
                                                "3des-cbc",
                                                "aes-128-cbc",
                                                "aes-192-cbc",
                                                "aes-256-cbc",
                                                "null",
                                            ],
                                        },
                                        "spi": {"type": "int"},
                                        "passphrase": {"type": "str"},
                                    },
                                },
                                "nssa": {
                                    "type": "dict",
                                    "options": {
                                        "translate": {"type": "bool"},
                                        "default_information_originate": {
                                            "type": "dict",
                                            "options": {
                                                "metric_type": {"type": "int"},
                                                "metric": {"type": "int"},
                                                "nssa_only": {"type": "bool"},
                                                "set": {"type": "bool"},
                                            },
                                        },
                                        "nssa_only": {"type": "bool"},
                                        "set": {"type": "bool"},
                                        "no_summary": {"type": "bool"},
                                    },
                                },
                                "stub": {
                                    "type": "dict",
                                    "options": {
                                        "summary_lsa": {"type": "bool"},
                                        "set": {"type": "bool"},
                                    },
                                },
                                "default_cost": {"type": "int"},
                                "authentication": {
                                    "type": "dict",
                                    "options": {
                                        "hidden_key": {"type": "bool"},
                                        "key": {"type": "str"},
                                        "algorithm": {
                                            "type": "str",
                                            "choices": ["md5", "sha1"],
                                        },
                                        "encrypt_key": {"type": "bool"},
                                        "spi": {"type": "int"},
                                        "passphrase": {"type": "str"},
                                    },
                                },
                            },
                        },
                        "address_family": {
                            "elements": "dict",
                            "type": "list",
                            "options": {
                                "router_id": {"type": "str"},
                                "distance": {"type": "int"},
                                "redistribute": {
                                    "elements": "dict",
                                    "type": "list",
                                    "options": {
                                        "routes": {
                                            "type": "str",
                                            "choices": [
                                                "bgp",
                                                "connected",
                                                "static",
                                            ],
                                        },
                                        "route_map": {"type": "str"},
                                    },
                                },
                                "default_information": {
                                    "type": "dict",
                                    "options": {
                                        "metric_type": {"type": "int"},
                                        "always": {"type": "bool"},
                                        "metric": {"type": "int"},
                                        "originate": {"type": "bool"},
                                        "route_map": {"type": "str"},
                                    },
                                },
                                "afi": {
                                    "choices": ["ipv4", "ipv6"],
                                    "type": "str",
                                },
                                "fips_restrictions": {"type": "bool"},
                                "default_metric": {"type": "int"},
                                "maximum_paths": {"type": "int"},
                                "adjacency": {
                                    "type": "dict",
                                    "options": {
                                        "exchange_start": {
                                            "type": "dict",
                                            "options": {
                                                "threshold": {"type": "int"}
                                            },
                                        }
                                    },
                                },
                                "max_metric": {
                                    "type": "dict",
                                    "options": {
                                        "router_lsa": {
                                            "type": "dict",
                                            "options": {
                                                "external_lsa": {
                                                    "type": "dict",
                                                    "options": {
                                                        "set": {
                                                            "type": "bool"
                                                        },
                                                        "max_metric_value": {
                                                            "type": "int"
                                                        },
                                                    },
                                                },
                                                "summary_lsa": {
                                                    "type": "dict",
                                                    "options": {
                                                        "set": {
                                                            "type": "bool"
                                                        },
                                                        "max_metric_value": {
                                                            "type": "int"
                                                        },
                                                    },
                                                },
                                                "set": {"type": "bool"},
                                                "on_startup": {
                                                    "type": "dict",
                                                    "options": {
                                                        "wait_for_bgp": {
                                                            "type": "bool"
                                                        },
                                                        "wait_period": {
                                                            "type": "int"
                                                        },
                                                    },
                                                },
                                                "include_stub": {
                                                    "type": "bool"
                                                },
                                            },
                                        }
                                    },
                                },
                                "log_adjacency_changes": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "detail": {"type": "bool"},
                                    },
                                },
                                "timers": {
                                    "type": "dict",
                                    "options": {
                                        "throttle": {
                                            "type": "dict",
                                            "options": {
                                                "max": {"type": "int"},
                                                "initial": {"type": "int"},
                                                "min": {"type": "int"},
                                                "spf": {"type": "bool"},
                                                "lsa": {"type": "bool"},
                                            },
                                        },
                                        "out_delay": {"type": "int"},
                                        "pacing": {"type": "int"},
                                        "lsa": {"type": "int"},
                                    },
                                },
                                "shutdown": {"type": "bool"},
                                "auto_cost": {
                                    "type": "dict",
                                    "options": {
                                        "reference_bandwidth": {"type": "int"}
                                    },
                                },
                                "graceful_restart_helper": {"type": "bool"},
                                "passive_interface": {"type": "bool"},
                                "bfd": {
                                    "type": "dict",
                                    "options": {
                                        "all_interfaces": {"type": "bool"}
                                    },
                                },
                                "areas": {
                                    "elements": "dict",
                                    "type": "list",
                                    "options": {
                                        "ranges": {
                                            "elements": "dict",
                                            "type": "list",
                                            "options": {
                                                "subnet_mask": {"type": "str"},
                                                "advertise": {"type": "bool"},
                                                "cost": {"type": "int"},
                                                "subnet_address": {
                                                    "type": "str"
                                                },
                                                "address": {"type": "str"},
                                            },
                                        },
                                        "area_id": {"type": "str"},
                                        "encryption": {
                                            "type": "dict",
                                            "options": {
                                                "hidden_key": {"type": "bool"},
                                                "key": {"type": "str"},
                                                "algorithm": {
                                                    "type": "str",
                                                    "choices": ["sha1", "md5"],
                                                },
                                                "encrypt_key": {
                                                    "type": "bool"
                                                },
                                                "encryption": {
                                                    "type": "str",
                                                    "choices": [
                                                        "3des-cbc",
                                                        "aes-128-cbc",
                                                        "aes-192-cbc",
                                                        "aes-256-cbc",
                                                        "null",
                                                    ],
                                                },
                                                "spi": {"type": "int"},
                                                "passphrase": {"type": "str"},
                                            },
                                        },
                                        "nssa": {
                                            "type": "dict",
                                            "options": {
                                                "translate": {"type": "bool"},
                                                "default_information_originate": {
                                                    "type": "dict",
                                                    "options": {
                                                        "metric_type": {
                                                            "type": "int"
                                                        },
                                                        "metric": {
                                                            "type": "int"
                                                        },
                                                        "nssa_only": {
                                                            "type": "bool"
                                                        },
                                                        "set": {
                                                            "type": "bool"
                                                        },
                                                    },
                                                },
                                                "nssa_only": {"type": "bool"},
                                                "set": {"type": "bool"},
                                                "no_summary": {"type": "bool"},
                                            },
                                        },
                                        "stub": {
                                            "type": "dict",
                                            "options": {
                                                "summary_lsa": {
                                                    "type": "bool"
                                                },
                                                "set": {"type": "bool"},
                                            },
                                        },
                                        "default_cost": {"type": "int"},
                                        "authentication": {
                                            "type": "dict",
                                            "options": {
                                                "hidden_key": {"type": "bool"},
                                                "key": {"type": "str"},
                                                "algorithm": {
                                                    "type": "str",
                                                    "choices": ["md5", "sha1"],
                                                },
                                                "encrypt_key": {
                                                    "type": "bool"
                                                },
                                                "spi": {"type": "int"},
                                                "passphrase": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "graceful_restart": {
                                    "type": "dict",
                                    "options": {
                                        "grace_period": {"type": "int"},
                                        "set": {"type": "bool"},
                                    },
                                },
                            },
                        },
                    },
                }
            },
        },
    }  # pylint: disable=C0301



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type
import re
import socket

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    to_list,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.providers.providers import (
    CliProvider,
)


class Neighbors(CliProvider):
    def render(self, config=None):
        commands = list()
        safe_list = list()

        router_context = "router bgp %s" % self.get_value("config.bgp_as")
        context_config = None

        for item in self.get_value("config.neighbors"):
            context_commands = list()

            neighbor = item["neighbor"]

            try:
                socket.inet_aton(neighbor)
                context = "neighbor %s" % neighbor
            except socket.error:
                context = "neighbor-group %s" % neighbor

            if config:
                context_path = [router_context, context]
                context_config = self.get_config_context(
                    config, context_path, indent=1
                )

            for key, value in iteritems(item):
                if value is not None:
                    meth = getattr(self, "_render_%s" % key, None)
                    if meth:
                        resp = meth(item, context_config)
                        if resp:
                            context_commands.extend(to_list(resp))

            if context_commands:
                commands.append(context)
                commands.extend(context_commands)
                commands.append("exit")

            safe_list.append(context)

        if config and safe_list:
            commands.extend(self._negate_config(config, safe_list))

        return commands

    def _negate_config(self, config, safe_list=None):
        commands = list()
        matches = re.findall(r"(neighbor \S+)", config, re.M)
        for item in set(matches).difference(safe_list):
            commands.append("no %s" % item)
        return commands

    def _render_remote_as(self, item, config=None):
        cmd = "remote-as %s" % item["remote_as"]
        if not config or cmd not in config:
            return cmd

    def _render_description(self, item, config=None):
        cmd = "description %s" % item["description"]
        if not config or cmd not in config:
            return cmd

    def _render_enabled(self, item, config=None):
        cmd = "shutdown"
        if item["enabled"] is True:
            cmd = "no %s" % cmd
        if not config or cmd not in config:
            return cmd

    def _render_update_source(self, item, config=None):
        cmd = "update-source %s" % item["update_source"].replace(" ", "")
        if not config or cmd not in config:
            return cmd

    def _render_password(self, item, config=None):
        cmd = "password %s" % item["password"]
        if not config or cmd not in config:
            return cmd

    def _render_ebgp_multihop(self, item, config=None):
        cmd = "ebgp-multihop %s" % item["ebgp_multihop"]
        if not config or cmd not in config:
            return cmd

    def _render_tcp_mss(self, item, config=None):
        cmd = "tcp mss %s" % item["tcp_mss"]
        if not config or cmd not in config:
            return cmd

    def _render_advertisement_interval(self, item, config=None):
        cmd = "advertisement-interval %s" % item["advertisement_interval"]
        if not config or cmd not in config:
            return cmd

    def _render_neighbor_group(self, item, config=None):
        cmd = "use neighbor-group %s" % item["neighbor_group"]
        if not config or cmd not in config:
            return cmd

    def _render_timers(self, item, config):
        """generate bgp timer related configuration
        """
        keepalive = item["timers"]["keepalive"]
        holdtime = item["timers"]["holdtime"]
        min_neighbor_holdtime = item["timers"]["min_neighbor_holdtime"]

        if keepalive and holdtime:
            cmd = "timers %s %s" % (keepalive, holdtime)
            if min_neighbor_holdtime:
                cmd += " %s" % min_neighbor_holdtime
            if not config or cmd not in config:
                return cmd
        else:
            raise ValueError(
                "required both options for timers: keepalive and holdtime"
            )



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

"""
The module file for iosxr_interfaces
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type
DOCUMENTATION = """
module: iosxr_interfaces
short_description: Interfaces resource module
description: This module manages the interface attributes on Cisco IOS-XR network
  devices.
version_added: 1.0.0
author:
- Sumit Jaiswal (@justjais)
- Rohit Thakur (@rohitthakur2590)
notes:
- Tested against Cisco IOS-XRv Version 6.1.3 on VIRL.
- This module works with connection C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html).
options:
  config:
    description: A dictionary of interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Full name of the interface to configure in C(type + path) format. e.g. C(GigabitEthernet0/0/0/0)
        type: str
        required: true
      description:
        description:
        - Interface description.
        type: str
      enabled:
        default: true
        description:
        - Administrative state of the interface.
        - Set the value to C(True) to administratively enable the interface or C(False)
          to disable it.
        type: bool
      speed:
        description:
        - Configure the speed for an interface. Default is auto-negotiation when not
          configured.
        type: int
      mtu:
        description:
        - Sets the MTU value for the interface. Applicable for Ethernet interfaces
          only.
        - Refer to vendor documentation for valid values.
        type: int
      duplex:
        description:
        - Configures the interface duplex mode. Default is auto-negotiation when not
          configured.
        type: str
        choices:
        - full
        - half
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the IOS-XR device
      by executing the command B(show running-config interface).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
    - merged
    - parsed
    - deleted
    - replaced
    - rendered
    - gathered
    - overridden
    default: merged
    description:
    - The state of the configuration after module completion
    type: str
"""

EXAMPLES = """
# Using merged
# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  description Replaced by Ansible Team
#  mtu 2000
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  dot1q native vlan 1021
# !
- name: Configure Ethernet interfaces
  cisco.iosxr.iosxr_interfaces:
    config:
    - name: GigabitEthernet0/0/0/2
      description: Configured by Ansible
      enabled: true
    - name: GigabitEthernet0/0/0/3
      description: Configured by Ansible Network
      enabled: false
      duplex: full
    state: merged
# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  description Configured and Merged by Ansible Network
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  description Configured and Merged by Ansible Network
#  mtu 2600
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex full
#  shutdown
#  dot1q native vlan 1021
# !
# Using replaced
# Before state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  description Configured by Ansible
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  description Test
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  dot1q native vlan 1021
# !
- name: Configure following interfaces and replace their existing config
  cisco.iosxr.iosxr_interfaces:
    config:
    - name: GigabitEthernet0/0/0/2
      description: Configured by Ansible
      enabled: true
      mtu: 2000
    - name: GigabitEthernet0/0/0/3
      description: Configured by Ansible Network
      enabled: false
      duplex: auto
    state: replaced
# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  description Configured by Ansible
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  description Configured and Replaced by Ansible
#  mtu 2000
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  description Configured and Replaced by Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 1021
# !
# Using overridden
# Before state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  description Configured by Ansible
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  description Configured by Ansible
#  mtu 2600
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex full
#  shutdown
#  dot1q native vlan 1021
# !
- name: Override interfaces
  cisco.iosxr.iosxr_interfaces:
    config:
    - name: GigabitEthernet0/0/0/2
      description: Configured by Ansible
      enabled: true
      duplex: auto
    - name: GigabitEthernet0/0/0/3
      description: Configured by Ansible Network
      enabled: false
      speed: 1000
    state: overridden
# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  description Configured and Overridden by Ansible Network
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  speed 1000
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  description Configured and Overridden by Ansible Network
#  mtu 2000
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex full
#  shutdown
#  dot1q native vlan 1021
# !
# Using deleted
# Before state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  description Configured and Overridden by Ansible Network
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  speed 1000
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  description Configured and Overridden by Ansible Network
#  mtu 2000
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex full
#  shutdown
#  dot1q native vlan 1021
# !
- name: Delete IOSXR interfaces as in given arguments
  cisco.iosxr.iosxr_interfaces:
    config:
    - name: GigabitEthernet0/0/0/2
    - name: GigabitEthernet0/0/0/3
    state: deleted
# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf custB
#  ipv4 address 178.18.169.23 255.255.255.0
#  dot1q native vlan 30
# !
# interface GigabitEthernet0/0/0/3
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  dot1q native vlan 1021
# !
# Using parsed
# parsed.cfg
# ------------
#
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
#  description Configured and Merged by Ansible-Network
#  mtu 110
#  ipv4 address 172.31.1.1 255.255.255.0
#  duplex half
# !
# interface GigabitEthernet0/0/0/3
#  shutdown
# !
# interface GigabitEthernet0/0/0/4
#  shutdown
# !
# - name: Convert ACL interfaces config to argspec without connecting to the appliance
#   cisco.iosxr.iosxr_interfaces:
#     running_config: "{{ lookup('file', './parsed.cfg') }}"
#     state: parsed
# Task Output (redacted)
# -----------------------
# "parsed": [
#        {
#            "name": "MgmtEth0/RP0/CPU0/0"
#        },
#        {
#            "access_groups": [
#                {
#                    "acls": [
#                        {
#                            "direction": "in",
#                            "name": "acl_1"
#                        },
#                        {
#                            "direction": "out",
#                            "name": "acl_2"
#                        }
#                    ],
#                    "afi": "ipv4"
#                },
#                {
#                    "acls": [
#                        {
#                            "direction": "in",
#                            "name": "acl6_1"
#                        },
#                        {
#                            "direction": "out",
#                            "name": "acl6_2"
#                        }
#                    ],
#                    "afi": "ipv6"
#                }
#            ],
#            "name": "GigabitEthernet0/0/0/0"
#        },
#        {
#            "access_groups": [
#                {
#                    "acls": [
#                        {
#                            "direction": "out",
#                            "name": "acl_1"
#                        }
#                    ],
#                    "afi": "ipv4"
#                }
#            ],
#            "name": "GigabitEthernet0/0/0/1"
#        }
#    ]
# }
# Using rendered
- name: Render platform specific commands from task input using rendered state
  cisco.iosxr.iosxr_interfaces:
    config:
    - name: GigabitEthernet0/0/0/0
      description: Configured and Merged by Ansible-Network
      mtu: 110
      enabled: true
      duplex: half
    - name: GigabitEthernet0/0/0/1
      description: Configured and Merged by Ansible-Network
      mtu: 2800
      enabled: false
      speed: 100
      duplex: full
    state: rendered
# Task Output (redacted)
# -----------------------
# "rendered": [
#         "interface GigabitEthernet0/0/0/0",
#         "description Configured and Merged by Ansible-Network",
#         "mtu 110",
#         "duplex half",
#         "no shutdown",
#         "interface GigabitEthernet0/0/0/1",
#         "description Configured and Merged by Ansible-Network",
#         "mtu 2800",
#         "speed 100",
#         "duplex full",
#         "shutdown"
#     ]
# Using gathered
# Before state:
# ------------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config  interface
# interface Loopback888
# description test for ansible
# shutdown
# !
# interface MgmtEth0/0/CPU0/0
# ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
# description Configured and Merged by Ansible-Network
# mtu 110
# ipv4 address 172.31.1.1 255.255.255.0
# duplex half
# !
# interface GigabitEthernet0/0/0/3
# shutdown
# !
# interface GigabitEthernet0/0/0/4
# shutdown
# !
- name: Gather IOSXR interfaces as in given arguments
  cisco.iosxr.iosxr_interfaces:
    config:
    state: gathered
# Task Output (redacted)
# -----------------------
#
# "gathered": [
#         {
#             "description": "test for ansible",
#             "enabled": false,
#             "name": "Loopback888"
#         },
#         {
#             "description": "Configured and Merged by Ansible-Network",
#             "duplex": "half",
#             "enabled": true,
#             "mtu": 110,
#             "name": "GigabitEthernet0/0/0/0"
#         },
#         {
#             "enabled": false,
#             "name": "GigabitEthernet0/0/0/3"
#         },
#         {
#             "enabled": false,
#             "name": "GigabitEthernet0/0/0/4"
#         }
#     ]
# After state:
# ------------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config  interface
# interface Loopback888
# description test for ansible
# shutdown
# !
# interface MgmtEth0/0/CPU0/0
# ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
# description Configured and Merged by Ansible-Network
# mtu 110
# ipv4 address 172.31.1.1 255.255.255.0
# duplex half
# !
# interface GigabitEthernet0/0/0/3
# shutdown
# !
# interface GigabitEthernet0/0/0/4
# shutdown
# !
"""

RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: The configuration returned will always be in the same format of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: The configuration returned will always be in the same format of the parameters above.
commands:
  description: The set of commands pushed to the remote device
  returned: always
  type: list
  sample: ['interface GigabitEthernet0/0/0/2', 'description: Configured by Ansible', 'shutdown']
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.interfaces.interfaces import (
    InterfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.interfaces.interfaces import (
    Interfaces,
)


def main():
    """
    Main entry point for module execution
    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]
    module = AnsibleModule(
        argument_spec=InterfacesArgs.argument_spec,
        required_if=required_if,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    result = Interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()
